# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

import string, time, os, webbrowser
import language, events, ui, host
import sb.aodb as ao
import sb.addon
import sb.profdb as pr
import sb.confdb as st
import sb.util.dialog


# An instance of the message timer.
helpTextTimer = ui.Timer('show-help-text-now')

# A widget will be created during initialization.
helpText = None

# This is true if the help panel shouldn't be updated at the moment.
helpDisabled = False

# The Snowberry logo in the bottom of the help panel.
logo = None

# The normal help text is composed of several fields.
FIELD_MAIN = 0
FIELD_CURRENT_TAB = 1
FIELD_CURRENT_SETTING = 2
FIELD_COMMAND = 3

helpFields = ['', '', '', '']

detailedAddonMode = False
currentAddon = None
currentSetting = None


def init():
    "Create the HTML text widget into the help area."
    
    ui.addMenuCommand(ui.MENU_HELP, 'open-documentation', pos=0)
    
    try:
        helpArea = ui.getArea(ui.HELP)
    except KeyError:
        # The Help area does not exist. We don't have much to do here.
        events.addCommandListener(handleCommand, ['open-documentation'])
        return
        
    helpArea.setExpanding(True)
    helpArea.setWeight(1)

    # Create a HTML text widget.
    global helpText
    helpArea.setBorder(3)
    helpText = helpArea.createFormattedText()

    # Unfreeze the help text after a minor delay. This'll make the app
    # init a bit smoother.
    helpText.freeze()
    helpTextTimer.start(1000)

    # Set parameters suitable for the logo.
    helpArea.setWeight(0)
    helpArea.setBorder(0)

    # Register a listener.
    events.addCommandListener(handleCommand, ['help-addon-mode-brief',
                                              'help-addon-mode-detailed',
                                              'freeze', 'unfreeze',
                                              'open-documentation'])

    events.addNotifyListener(handleNotify, ['show-help-text-now',
                                            'init-done',
                                            'active-profile-changed',
                                            'tab-selected',
                                            'addon-list-selected',
                                            'maps-list-selected',
                                            'focus-changed',
                                            'value-changed',
                                            'language-changed'] )


def showLogo(doShow=True):
    """Show or hide the Snowberry logo in the bottom of the help panel.

    @param doShow True, if the logo should be shown.
    """
    global logo

    try:
        helpArea = ui.getArea(ui.HELP)
    except KeyError:
        # No Help area.
        return

    if not logo and doShow:
        logo = helpArea.createImage('help-logo')
        helpArea.updateLayout()

    elif logo and not doShow:
        helpArea.destroyWidget(logo)
        logo = None
        helpArea.updateLayout()
    

def handleCommand(event):
    """This is called when somebody sends a command.

    @param event A events.Command object.
    """
    global detailedAddonMode
    global helpDisabled
    
    if event.hasId('help-addon-mode-brief'):
        detailedAddonMode = False
        if currentAddon:
            showAddonInfo(currentAddon)

    elif event.hasId('help-addon-mode-detailed'):
        detailedAddonMode = True
        if currentAddon:
            showAddonInfo(currentAddon)

    elif event.hasId('freeze'):
        helpDisabled = True

    elif event.hasId('unfreeze'):
        helpDisabled = False
        updateHelpText()    
        
    elif event.hasId('open-documentation'):
        webbrowser.open_new(language.translate('documentation-url'))
            

def handleNotify(event):
    """This is called when somebody sends a notification.

    @param event A events.Notify object.
    """
    if event.hasId('show-help-text-now'):
        helpText.unfreeze()    
        return
    
    if helpDisabled:
        return
    
    if event.hasId('init-done'):
        setField(FIELD_MAIN, language.translate('help-welcome'))
        setField(FIELD_COMMAND, language.translate('help-command-defaults'))
        updateHelpText()    

    elif event.hasId('show-help-text-now'):
        helpText.unfreeze()
                        
    elif event.hasId('active-profile-changed'):
        if pr.getActive() is pr.getDefaults():
            setField(FIELD_COMMAND,
                     language.translate('help-command-defaults'))
        else:
            setField(FIELD_COMMAND, language.translate('help-command'))

    elif event.hasId('tab-selected'):
        setField(FIELD_MAIN, '')
        setField(FIELD_CURRENT_TAB,
                 language.translate('help-' + event.getSelection()))

    elif event.hasId('addon-list-selected') or \
         event.hasId('maps-list-selected'):
        # Display information about the selected addon in the panel.
        try:
            showAddonInfo(ao.get(event.getSelection()))
        except KeyError:
            # It wasn't an addon.
            pass

    elif event.hasId('focus-changed'): 
        try:
            setting = st.getSetting(event.getFocus())
            showSettingInfo(setting)
        except KeyError:
            # It was likely not a setting id.
            pass

    elif event.hasId('value-changed'):
        if currentSetting and event.getSetting() == currentSetting.getId():
            showSettingInfo(currentSetting)

    elif event.hasId('language-changed'):
        pass
        

def setField(which, text):
    global helpFields
    helpFields[which] = text
    updateHelpText()


def updateHelpText():
    """Update the help text with the standard fields.  The values of
    the fields depend on the currently selected tabs and settings.
    """
    msg = '<html><table width="100%" height="100%" border="0" ' + \
          'cellspacing="0" cellpadding="6"><tr>' + \
          '<td align="left">'
    msg += string.join(helpFields, "<p>") + "</table></html>"
    helpText.setText(msg)
    showLogo()

    # No longer showing a setting.
    global currentSetting
    currentSetting = None


# Background color for table header entries.
bgColor = "#E8E8E8"
tableRow = '<tr>'
tableBegin = '<table width="100%" cellspacing="1" cellpadding="2">'
tableEnd = '</table><p>'


def entryHeader(text, span=1):
    htmlSize = st.getSystemInteger('style-html-size') - 1
    width = ''
    return '<td %s bgcolor="%s" valign="top" colspan="%i"><font size="%i">%s</font>' % (width, bgColor, span, htmlSize, text)

def entryContent(text, span=1):
    htmlSize = st.getSystemInteger('style-html-size') - 1
    return '<td valign="top" colspan="%i"><font size="%i">%s</font>' \
           % (span, htmlSize, text)


def showAddonInfo(addon):
    """Display a summary of an addon in the help panel.

    @param addon An addons.Addon object.
    """
    global currentAddon, currentSetting

    # No longer showing a setting.
    currentSetting = None

    # The current addon that is displayed in the help.
    currentAddon = addon

    oneLiners = (detailedAddonMode == False)
    ident = addon.getId()
    
    # Begin the page with a table that contains the basic info.
    msg = '<html>'

    def optLink(id, isLink):
        if isLink:
            return '<a href="%s">%s</a>' % (id, language.translate(id))
        else:
            return language.translate(id)

    # The detail selection links.
    msg += '<center><font size="-1">'
    msg += optLink('help-addon-mode-brief', detailedAddonMode) + ' | '
    msg += optLink('help-addon-mode-detailed', not detailedAddonMode)
    msg += '</font></center><br>'

    # Summary table.
    msg += tableBegin

    msg += '<tr><td colspan="2"><b><font size="+1">' + \
           language.translate(ident) + \
           '</font></b>'

    def makeField(msg, fieldId, oneLiner=True):
        title = language.translate('help-addon-' + fieldId)
        contentId = ident + '-' + fieldId

        text = ''
        if language.isDefined(contentId):
            # This information has been defined.
            text = language.translate(contentId)
        elif detailedAddonMode:
            # Force all fields to show.
            text = '-'

        if text:
            if oneLiner:
                msg += tableRow + entryHeader(title) + entryContent(text)
            else:
                msg += tableRow + entryHeader(title, 2) + \
                       tableRow + entryContent(text, 2)

        return msg

    msg = makeField(msg, 'version')

    if detailedAddonMode:
        # Get the last modification time from the addon.
        modTime = time.localtime(addon.getLastModified())
        msg += tableRow + entryHeader(language.translate('help-addon-date')) \
               + entryContent(time.strftime("%d %b %Y", modTime))
    
    msg = makeField(msg, 'summary', oneLiners)

    if detailedAddonMode:
        msg = makeField(msg, 'contact', oneLiners)
        msg = makeField(msg, 'author', oneLiners)
        msg = makeField(msg, 'copyright', oneLiners)
        msg = makeField(msg, 'license', oneLiners)

    # Dependencies.
    deps = []
    
    exCats = addon.getExcludedCategories()
    if len(exCats):
        deps.append((language.translate('help-addon-excluded-categories'),
                     map(lambda c: language.translate(c.getLongId()), exCats)))

    for type, label in [(sb.addon.EXCLUDES, 'help-addon-excludes'),
                        (sb.addon.REQUIRES, 'help-addon-requires'),
                        (sb.addon.PROVIDES, 'help-addon-provides'),
                        (sb.addon.OFFERS, 'help-addon-offers')]:
        # Make a copy of the list so we can modify it.
        keywords = [kw for kw in addon.getKeywords(type)]

        # In the Brief mode, hide the identifier of the addon in the
        # Provides field.
        if not detailedAddonMode and type == sb.addon.PROVIDES:
            keywords.remove(addon.getId())            
        
        if len(keywords) > 0:
            # Include this in the info.
            def xlate(key):
                if ao.exists(key):
                    return '<a href="%s">%s</a>' % \
                           (key, language.translate(key))
                else:
                    return key
            deps.append((language.translate(label), map(xlate, keywords)))

    if len(deps):
        # Print a dependencies table.
        msg += tableRow + entryHeader(
            language.translate('help-addon-dependencies'), 2)

        content = ""

        for dep in deps:
            if dep is not deps[0]:
                content += '<br>'
            content += "<b>" + dep[0] + ":</b><ul><li>"
            content += string.join(dep[1], '<li>')
            content += '</ul>'

        msg += tableRow + entryContent(content, 2)            

    msg += tableEnd

    # Inside a box?
    if addon.getBox():
        box = addon.getBox()
        msg += language.expand(
            language.translate('help-addon-part-of-box'),
            box.getId(), language.translate(box.getId())) + '<p>'

    # The overview.
    if language.isDefined(ident + '-readme'):
        msg += language.translate(ident + '-readme')

    msg += '</html>'

    helpText.setText(msg)
    showLogo(False)


def showSettingInfo(setting):
    """Display information about the focused setting in the help
    panel.

    @param setting A settings.Setting object.
    """
    # Values will be taken from the active profile.
    prof = pr.getActive()

    # The identifier of the setting.
    ident = setting.getId()
    
    msg = '<html>'

    # The name of the setting.
    msg += '<b><font size="+1">' + language.translate(ident) + \
           '</font></b>'

    # The command line option.
    msg += '<br><table bgcolor="' + bgColor + '" width="100%"><tr><td>' + \
           '<font size="-1"><tt>' + \
           setting.getCommandLine(prof) + '</tt></font></table><p>'

    fromDefaults = False

    if prof.getValue(ident, False) == None:
        fromDefaults = True
        # The value comes from the default profile.
        msg += language.expand(
            language.translate('help-value-from-defaults'),
            pr.getDefaults().getName()) + '<p>'

    def valueToText(v):
        if language.isDefined(v):
            return language.translate(v)
        else:
            return v

    msg += tableBegin

    # The current value of the setting.
    value = prof.getValue(ident)
    if value:
        #msg += '<b>%s:</b><br>' % language.translate('help-value-current')
        #msg += valueToText(value.getValue())
        #msg += '<p>'

        msg += tableRow + \
               entryHeader(language.translate('help-value-current')) + \
               entryContent(valueToText(value.getValue()))

    # Min and max for the range and slider settings.
    if setting.getType() == 'slider' or setting.getType() == 'range':
        #msg += '<b>' + language.translate('help-value-min') + ':</b><br>' + \
        #       str(setting.getMinimum())
        #msg += '<p><b>' + language.translate('help-value-max') + \
        #       ':</b><br>' + str(setting.getMaximum())
        #msg += '<p>'

        msg += tableRow + entryHeader(language.translate('help-value-min')) + \
               entryContent(str(setting.getMinimum())) + \
               tableRow + entryHeader(language.translate('help-value-max')) + \
               entryContent(str(setting.getMaximum()))

    # The default.
    if prof.getValue(ident, False) != None and prof is pr.getDefaults():
        if setting.getDefault() != None:
            #msg += '<b>%s:</b><br>' % language.translate('help-value-default')
            #msg += valueToText(str(setting.getDefault()))
            #msg += '<p>'

            msg += tableRow + \
                   entryHeader(language.translate('help-value-default')) + \
                   entryContent(valueToText(str(setting.getDefault())))
            
    elif not fromDefaults and prof is not pr.getDefaults():
        defValue = pr.getDefaults().getValue(ident)
        if defValue:
            #msg += '<b>%s:</b><br>' % language.translate('help-value-default')
            #msg += valueToText(defValue.getValue())
            #msg += '<p>'

            msg += tableRow + \
                   entryHeader(language.translate('help-value-default')) + \
                   entryContent(valueToText(defValue.getValue()))

    msg += tableEnd

    # The help text of this setting.
    helpId = ident + '-help'
    if language.isDefined(helpId):
        msg += language.translate(helpId)

    msg += '</html>'

    # Display the setting information in the help panel.
    helpText.setText(msg)
    showLogo(False)

    # The info will be automatically updated in case it changes while
    # displayed.
    global currentSetting
    currentSetting = setting



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2006
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file tab.py Tab Area Widgets

import wx
import ui, host, events, language, paths
import sb.profdb as pr
import widgets as wg
import base
from list import FormattedList


class FormattedTabArea (FormattedList):
    """FormattedTabArea extends FormattedList by pairing it up with a
    previously created MultiArea.  This class implements the same
    interface as TabArea.

    This widget class is different from the other classes.  The other
    classes encapsulate a single wxWidgets widget.  FormattedTabArea
    combines a FormattedList with a MultiArea object."""

    def __init__(self, parent, wxId, id):
        """Constuct a new formatted tab area.

        @param parent The parent wxPanel.

        @param wxId The ID number of the formatted list.

        @param id The identifier of the formatted list.

        @param ownerArea The Area in which the formatted tab area
        resides.
        """
        FormattedList.__init__(self, parent, wxId, id,
                               style=wx.SUNKEN_BORDER)

        self.setSorted(False)
        self.multi = None

        # List of all currently hidden tabs.
        self.hiddenTabs = []

    def setMultiArea(self, multiArea):
        """Set the MultiArea that will react to the changes in the list.

        @param multiArea The ui.MultiArea object to pair up with the
        list.
        """
        self.multi = multiArea

    def __getPresentation(self, tabId):
        """Compose the HTML presentation for a tab identifier.

        @return HTML content as a string.
        """
        # Kludge for the game-options icon.  There should be a way to
        # map the icon more flexibly so this hardcoding wouldn't be
        # needed.
        if tabId == 'game-options':
            game = 'game-jdoom'
            if pr.getActive():
                for c in pr.getActive().getComponents():
                    if c[:5] == 'game-':
                        game = c
                        break
            imageName = language.translate(game + '-icon')
        else:
            imageName = language.translate(tabId + '-icon')

        return ('<table width="100%" border=0 cellspacing=3 cellpadding=1>' +
                '<tr><td width=35><img width=32 height=32 ' +
                'src="%s"></td><td align="left" valign="center">%s</td>' %
                (paths.findBitmap(imageName), language.translate(tabId)) +
                '</tr></table>')

    def retranslate(self):
        """Update the icons and names of the categories."""

        for tab in self.getTabs():
            self.updateIcon(tab)

        self.multi.refresh()

    def getTabs(self):
        """Compose a list of all the areas of the tab area.  Hidden
        areas are included, too.

        @return An array that contains identifiers.
        """
        return self.multi.getPages()

    def getSelectedTab(self):
        """Returns the identifier of the currently selected tab.
        Returns None if no tab is selected."""

        return self.getSelectedItem()

    def addTab(self, id):
        """Add a new tab into the TabArea widget.  The tab is appended
        after previously added tabs.

        @param id Identifier for the tab.  The name and the icon of
        the tab will be determined using this identifier.

        @return The Area object for the contents of the tab.
        """
        area = self.multi.createPage(id)

        # Add it to the end.
        self.addItem(id, self.__getPresentation(id))

        # Automatically show the first page.
        if self.getItemCount() == 1:
            self.selectTab(id)

        return area

    def removeAllTabs(self):
        """Remove all tabs from the tab area."""

        for id in self.getTabs():
            self.removeTab(id)

    def removeTab(self, id):
        """Remove an existing tab from the TabArea widget.

        @param id Identifier of the tab to remove.
        """
        if id in self.hiddenTabs:
            self.hiddenTabs.remove(id)

        self.multi.removePage(id)
        self.removeItem(id)

    def selectTab(self, selectedId):
        """Select a specific tab.

        @param selectedId Identifier of the tab to select.
        """
        self.selectItem(selectedId)
        self.multi.showPage(selectedId)

    def hideTab(self, identifier):
        """Hide a tab in the tab area.

        @param identifier Identifier of the tab to hide.
        """
        self.showTab(identifier, False)

    def showTab(self, identifier, doShow=True):
        """Show a hidden tab in the tab area, or hide a tab.  Hiding
        and showing does not alter the order of the tabs.

        @param identifier Identifier of the tab to show or hide.

        @param doShow True, if the tab should be shown; False, if hidden.
        """
        if doShow and identifier in self.hiddenTabs:
            self.hiddenTabs.remove(identifier)
            self.addItem(identifier, self.__getPresentation(identifier), 0)

        elif not doShow and identifier not in self.hiddenTabs:
            # Move the selection to the previous item.
            #if self.getSelectedTab() == identifier:
            #    index = min(self.getItemCount() - 1, self.getSelectedIndex() + 1)
            #    self.selectTab(self.items[index][0])

            self.hiddenTabs.append(identifier)
            self.removeItem(identifier)

    def onItemSelected(self, ev):
        FormattedList.onItemSelected(self, ev)

        self.multi.showPage(self.getSelectedItem())

    def updateIcon(self, identifier):
        """Update the presentation of the specified tab.

        @param identifier Tab identifier.
        """
        self.addItem(identifier, self.__getPresentation(identifier))

#    def updateIcons(self):
#        """Update the items in the formatted list."""
#
#        #tabs = self.getTabs()
#        #self.addItem(




class TabArea (base.Widget):
    """TabArea is a widget that implements a tab area.  It supports a
    number of different styles: TabArea.STYLE_ICON and
    TabArea.STYLE_CHOICE.  In the icon style, the user can select the
    visible tab by clicking on one of the icons in the list.  In the
    choice style, there is drop-down list for the tab titles.
    """

    # Tab area styles.
    STYLE_ICON = 'icon'
    STYLE_HORIZ_ICON = 'horiz-icon'
    STYLE_DROP_LIST = 'drop'
    STYLE_BASIC = 'basic'
    STYLE_FORMATTED = 'formatted'

    def __init__(self, parent, wxId, id, style):
        # The formatted tab area style uses another widget entirely.
        if style == TabArea.STYLE_FORMATTED:
            raise Exception('not supported here')

        # Create the appropriate widget.
        if style == TabArea.STYLE_ICON:
            w = wx.Listbook(parent, wxId, style=wx.LB_LEFT)
            w.GetListView().SetSingleStyle(wx.LC_AUTOARRANGE)
        elif style == TabArea.STYLE_HORIZ_ICON:
            w = wx.Listbook(parent, wxId, style=wx.LB_TOP)
        elif style == TabArea.STYLE_BASIC:
            w = wx.Notebook(parent, wxId)
        else:
            w = wx.Choicebook(parent, wxId, style=wx.CHB_TOP)

        base.Widget.__init__(self, w)

        self.style = style

        # The horizontal icon style is handled like the normal icon
        # style.
        if self.style == TabArea.STYLE_HORIZ_ICON:
            self.style = TabArea.STYLE_ICON

        # The IconTabs widget remembers its id so it can send
        # notifications events.
        self.widgetId = id

        # The panel map is used when figuring out the IDs of each page
        # when an event occurs.  It contains an ordered array of
        # tuples: (identifier, panel).
        self.panelMap = []
        self.tabAreas = {}

        if self.style == TabArea.STYLE_ICON:
            wx.EVT_LISTBOOK_PAGE_CHANGED(parent, wxId, self.onTabChange)
            # The icon style uses the icons.
            w.SetImageList(wg.iconManager.getImageList())
        elif style == TabArea.STYLE_BASIC:
            wx.EVT_NOTEBOOK_PAGE_CHANGED(parent, wxId, self.onTabChange)
        else:
            wx.EVT_CHOICEBOOK_PAGE_CHANGED(parent, wxId, self.onTabChange)

    def clear(self):
        """When a TabArea is cleared, it must destroy all of its tabs,
        which contain subareas."""

        self.removeAllTabs()
        base.Widget.clear(self)

    def getTabs(self):
        """Compose a list of all the areas of the tab area.  Hidden
        areas are included, too.

        @return An array that contains identifiers.
        """
        return map(lambda mapped: mapped[0], self.panelMap)

    def __lookupPanel(self, panel):
        """Looks up a panel identifier in the panel map.

        @param panel A wxPanel object.

        @return Identifier of the tab.  None, if panel not found.
        """
        for id, mapped in self.panelMap:
            if panel == mapped:
                return id
        return None

    def __getTabIndex(self, id):
        """Determine the index of a specific tab in the tab area."""

        book = self.getWxWidget()
        for i in range(book.GetPageCount()):
            if self.__lookupPanel(book.GetPage(i)) == id:
                return i
        return None

    def __getPanelIndex(self, panel):
        """Determine the index of a specific panel in the tab area."""

        book = self.getWxWidget()
        for i in range(book.GetPageCount()):
            if book.GetPage(i) is panel:
                return i
        return None

    def addTab(self, id):
        """Add a new tab into the TabArea widget.  The tab is appended
        after previously added tabs.

        @param id Identifier for the tab.  The name and the icon of
        the tab will be determined using this identifier.

        @return The Area object for the contents of the tab.
        """
        book = self.getWxWidget()
        #tab = scrolled.ScrolledPanel(book, -1)
        #tab.SetupScrolling()
        tab = wx.Panel(book, -1, style=wx.CLIP_CHILDREN)

        if host.isWindows():
            tab.SetBackgroundStyle(wx.BG_STYLE_SYSTEM)

        # Put the new tab in the page map so that when an event
        # occurs, we will know the ID of the tab.
        self.panelMap.append((id, tab))

        # Insert the page into the book.
        book.AddPage(tab, language.translate(id))

        import sb.widget.area
        area = sb.widget.area.Area(id, tab, ui.ALIGN_VERTICAL)
        area.setExpanding(True)
        area.setWeight(1)

        # Store the Area object in a dictionary for quick access.
        self.tabAreas[id] = area

        self.updateIcons()

        return area

    def removeTab(self, id):
        """Remove an existing tab from the TabArea widget.

        @param id Identifier of the tab to remove.
        """
        panel = None
        for tabId, mapped in self.panelMap:
            if tabId == id:
                # This is the panel to remove.
                panel = mapped
                break

        if not panel:
            # It doesn't exist.
            return

        # Remove the panel from the map and destroy it.
        for i in range(len(self.panelMap)):
            if self.panelMap[i][1] is panel:
                del self.panelMap[i]
                break

        # Remove the page from the tab area, but this doesn't
        # destroy the panel.
        book = self.getWxWidget()
        for i in range(book.GetPageCount()):
            if book.GetPage(i) == panel:
                book.RemovePage(i)
                break

        self.tabAreas[id].destroy()
        del self.tabAreas[id]

    def removeAllTabs(self):
        """Remove all tabs from the tab area."""

        for id in self.getTabs():
            self.removeTab(id)

    def selectTab(self, selectedId):
        """Select a specific tab.

        @param id Tab identifier.
        """
        index = self.__getTabIndex(selectedId)
        if index != None:
            self.getWxWidget().SetSelection(index)

    def hideTab(self, identifier):
        """Hide a tab in the tab area.

        @param identifier Identifier of the tab to hide.
        """
        self.showTab(identifier, False)

    def showTab(self, identifier, doShow=True):
        """Show a hidden tab in the tab area, or hide a tab.  Hiding
        and showing does not alter the order of the tabs.

        @param identifier Identifier of the tab to show or hide.

        @param doShow True, if the tab should be shown; False, if hidden.
        """
        book = self.getWxWidget()

        if not doShow:
            # Hide the page by removing it from the widget.
            index = self.__getTabIndex(identifier)
            if index != None:
                # Is this currently selected?
                if index == book.GetSelection():
                    # Automatically select another page.
                    if index > 0:
                        book.SetSelection(index - 1)
                    elif index < book.GetPageCount() - 1:
                        book.SetSelection(index + 1)

                # This won't destroy the panel or its contents.
                book.RemovePage(index)

        else:
            # Show the page by inserting it to the correct index.
            for tabId, panel in self.panelMap:
                if tabId == identifier:
                    # Determine the location of the shown tab.
                    where = 0
                    for id, mapped in self.panelMap:
                        index = self.__getPanelIndex(mapped)
                        if id == identifier:
                            if index != None:
                                # The panel appears to be visible already.
                                return
                            break
                        if index != None:
                            where = index + 1

                    # Insert this panel back into the book.
                    book.InsertPage(where, panel,
                                    language.translate(identifier))
                    self.updateIcons()
                    return

    def updateIcons(self):
        """Update all the icons used by the tab area labels, if using
        an icon style.
        """
        if self.style == TabArea.STYLE_ICON:
            # Update all the tabs.
            for identifier, panel in self.panelMap:
                pageIndex = self.__getPanelIndex(panel)
                if pageIndex != None:
                    iconName = identifier + '-icon'
                    self.getWxWidget().SetPageImage(pageIndex,
                                                    wg.iconManager.get(iconName))

    def retranslate(self):
        """Retranslate the tab labels."""

        for identifier, panel in self.panelMap:
            pageIndex = self.__getPanelIndex(panel)
            if pageIndex != None:
                try:
                    self.getWxWidget().SetPageText(
                        pageIndex, language.translate(identifier))
                except:
                    # wxWidgets hasn't implemented the appropriate method?
                    pass

    def onTabChange(self, ev):
        """Handle the wxWidgets event that occurs when the currently
        selected tab changes."""

        # Figure out which panel is now visible.
        tabPanel = self.getWxWidget().GetPage(ev.GetSelection())

        # Send a notification event.
        notification = events.SelectNotify(self.widgetId,
                                           self.__lookupPanel(tabPanel))
        events.send(notification)



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2006
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file dialog.py Dialogs

import sys, os, wx, string
import wx.wizard as wiz
import ui, host, events, widgets, language
import sb.profdb as pr
import logger
from widgets import uniConv
import sb.widget.area
import sb.widget.button as wb


def chooseFile(prompt, default, mustExist, fileTypes, defExt=None):
    """Show a file chooser dialog.

    @param prompt Identifier of the dialog title string.

    @param default The default selection.

    @param mustExist The selected file must exist.

    @param fileTypes An array of tuples (name, extension).

    @return The full pathname of the selected file.
    """
    # Compile the string of file types.
    types = []
    for ident, ext in fileTypes:
        types.append(language.translate(ident))
        types.append('*.' + ext.lower() + ';*.' + ext.upper())

    # Always append the "all files" filter.
    types.append(language.translate('file-type-anything'))
    types.append('*.*')

    if not defExt:
        if len(fileTypes) > 0:
            # By default use the first file type.
            defExt = fileTypes[0][1]
        else:
            defExt = ''

    if mustExist:
        flags = wx.FILE_MUST_EXIST
    else:
        flags = 0

    selection = wx.FileSelector(language.translate(prompt),
                                os.path.dirname(default),
                                os.path.basename(default),
                                defExt,
                                string.join(types, '|'),
                                flags)
    return selection


def chooseFolder(prompt, default):
    """Show a directory chooser dialog.

    @param prompt Identifier of the dialog title string.

    @param default The default path.

    @return The selected path.
    """
    return wx.DirSelector(language.translate(prompt), default)


def createDialog(id, alignment=1, size=None, resizable=True):
    """Create an empty modal dialog box.

    @param id Identifier of the dialog area.  The title of the dialog
    is the translation of this identifier.

    @return A widgets.DialogArea object.
    """
    dialog = AreaDialog(ui.getMainPanel(), -1, id, size, alignment, resizable)
    return dialog


def createButtonDialog(id, buttons, defaultButton=None, size=None,
                       resizable=True):
    """Returns the area where the caller may add the contents of the
    dialog.

    @param id Identifier of the dialog.
    @param buttons An array of button commands.
    @param defaultButton The identifier of the default button.

    @return Tuple (dialog, user-area)
    """
    dialog = createDialog(id, ui.ALIGN_HORIZONTAL, size, resizable)
    area = dialog.getArea()

    area.setMinSize(400, 20)

    # The Snowberry logo is on the left.
    #area.setWeight(0)
    #imageArea = area.createArea(alignment=ui.ALIGN_VERTICAL, border=0,
    #                            backgroundColor=wx.Colour(255, 255, 255))
    #imageArea.setWeight(1)
    #imageArea.setExpanding(True)
    #imageArea.addSpacer()
    #imageArea.setWeight(0)
    #imageArea.createImage('snowberry')

    area.setWeight(1)
    area.setBorderDirs(ui.BORDER_NOT_BOTTOM)
    contentArea = area.createArea(alignment=ui.ALIGN_VERTICAL, border=6)
    contentArea.setWeight(0)

    # Generous borders.
    contentArea.setBorder(16)

    contentArea.setWeight(1)
    userArea = contentArea.createArea(border=6)

    # Create the buttons.
    contentArea.setWeight(0)
    contentArea.setBorderDirs(ui.BORDER_NOT_TOP)
    buttonArea = contentArea.createArea(alignment=ui.ALIGN_HORIZONTAL,
                                        border=0)

    # If no explicit spacing is defined, use the default right
    # alignment.
    if '' not in buttons:
        # The implied spacer.
        buttonArea.addSpacer()

    buttonArea.setBorder(6, ui.BORDER_LEFT_RIGHT)
    buttonArea.setWeight(0)

    if not host.isMac():
        # Follow the general guidelines of the platform.
        if '' in buttons:
            # Only reverse the portion after the ''.
            index = buttons.index('') + 1
            sub = buttons[index:]
            sub.reverse()
            buttons = buttons[:index] + sub
        else:
            buttons.reverse()

    for button in buttons:
        # If an empty identifier is given, insert a space here.
        if button == '':
            buttonArea.setWeight(1)
            buttonArea.addSpacer()
            buttonArea.setWeight(0)
            continue

        if button == defaultButton:
            style = wb.Button.STYLE_DEFAULT
        else:
            style = wb.Button.STYLE_NORMAL

        widget = buttonArea.createButton(button, style=style)
        dialog.identifyWidget(button, widget)

    # By default, focus the default button.
    if defaultButton:
        dialog.focusWidget(defaultButton)

    return dialog, userArea


class AreaDialog (wx.Dialog):
    """AreaDialog implements a wxDialog that has an Area inside it."""

    def __init__(self, parent, wxId, id, size, align=ui.ALIGN_VERTICAL, 
                 resizable=True):
                 
        # Determine flags.
        flags = wx.CLOSE_BOX | wx.CAPTION
        if resizable:
            flags |= wx.RESIZE_BORDER
                 
        wx.Dialog.__init__(self, parent, wxId, uniConv(language.translate(id)),
                           style=flags)
        self.dialogId = id
        self.widgetMap = {}

        if size:
            self.recommendedSize = size
        else:
            self.recommendedSize = None

        # All the commands that will cause the dialog to be closed.
        self.dialogEndCommands = ['ok', 'cancel', 'yes', 'no']

        # Create an area inside the dialog.
        self.area = sb.widget.area.Area('', self, alignment=align, border=0)

    def getArea(self):
        """Return the Area which contains the dialog's widgets.

        @return Area object.
        """
        return self.area

    def addEndCommand(self, command):
        """Add a new command that will close the dialog.

        @param command A command identifier.
        """
        self.dialogEndCommands.append(command)

    def center(self):
        """Position the dialog in the center of the screen."""

        self.Centre()

    def run(self, pos=None):
        """Show the dialog as modal.  Won't return until the dialog
        has been closed.

        @param pos  Tuple (x, y). Position for the dialog. Defaults to None.

        @return The command that closed the dialog.
        """
        self.area.getWxWidget().Fit(self)

        # We'll listen to the common dialog commands (ok, cancel)
        # ourselves.
        events.addCommandListener(self.handleCommand)

        # The command that closes the dialog.  We'll assume the dialog
        # gets closed.
        self.closingCommand = 'cancel'

        if self.recommendedSize is not None:
            self.SetSize(self.recommendedSize)

        if pos:
            self.MoveXY(*pos)
        else:
            self.Centre()

        # Won't return until the dialog closes.
        self.ShowModal()

        # Stop listening.
        events.removeCommandListener(self.handleCommand)

        return self.closingCommand

    def close(self, closingCommand):
        """Close a modal dialog box that is currently open.

        @param closingCommand The command that is reported as being
        the one that caused the dialog to close.
        """
        self.closingCommand = closingCommand
        self.EndModal(0)

    def handleCommand(self, event):
        """The dialog listens to commands while it is modal.

        @param event A events.Command object.
        """
        if event.getId() in self.dialogEndCommands:
            self.close(event.getId())

    def identifyWidget(self, identifier, widget):
        """Tell the dialog of a widget inside dialog.

        @param identifier Identifier of the widget.

        @param widget The widget object.
        """
        self.widgetMap[identifier] = widget

    def enableWidget(self, identifier, doEnable=True):
        try:
            self.widgetMap[identifier].enable(doEnable)
        except:
            pass

    def disableWidget(self, identifier):
        self.enableWidget(identifier, False)
        
    def focusWidget(self, identifier):
        """Focus a widget managed by the dialog.
        
        @param identifier  Identifier of the widget to focus."""
        
        try:
            self.widgetMap[identifier].focus()
        except:
            pass


class WizardDialog (wiz.Wizard):
    """A wizard dialog.  Automatically manages the Next and Prev
    buttons, and opening and closing of the dialog.  The pages of the
    dialog must be created separately, using WizardPages."""

    def __init__(self, title, imageFileName):
        bmp = wx.Bitmap(imageFileName)
        self.wxId = wx.NewId()
        wiz.Wizard.__init__(self, ui.getMainPanel(), self.wxId, title, bmp)

        self.SetBorder(0)
        
        self.pageReaction = None

        # List of all the WizardPage objects associated with this
        # wizard.  This list does NOT define the order of the pages,
        # just that they're owned by this wizard.
        self.ownedPages = []

        wiz.EVT_WIZARD_PAGE_CHANGED(ui.getMainPanel(), self.wxId, self.onPageChange)

    def setPageReaction(self, reaction):
        self.pageReaction = reaction

    def addOwnedPage(self, page):
        """Add a new page into the wizard.  This is done automatically
        when a new WizardPage is constructed.  Does not affect the
        order of the pages."""

        self.ownedPages.append(page)

    def onPageChange(self, event):
        pageId = event.GetPage().getId()
        events.send(events.SelectNotify('wizard', pageId))
        if self.pageReaction:
            self.pageReaction(event.GetPage())

    def run(self, firstPage):
        """Open the wizard dialog starting with the given page.

        @param firstPage A WizardPage object.
        """
        # Make sure the wizard window is large enough.
        self.FitToPage(firstPage)
        
        if self.RunWizard(firstPage):
            # The wizard was successful.
            return 'ok'
        else:
            return 'cancel'

    def destroy(self):
        # Destroy all the owned pages.
        for page in self.ownedPages:
            page.destroy()
        self.Destroy()


class WizardPage (wiz.PyWizardPage):
    """A wizard page that contains an Area for the widgets."""

    def __init__(self, wizard, title):
        wiz.PyWizardPage.__init__(self, wizard)

        # The title identifies the page.
        self.identifier = title

        # Links to other pages.
        self.prev = None
        self.next = None

        # Create an Area inside the page.
        paddingArea = sb.widget.area.Area('', self, alignment=ui.ALIGN_VERTICAL, border=12)
        self.area = paddingArea.createArea(alignment=ui.ALIGN_VERTICAL, border=6)
        self.area.setWeight(0)

        # Create the title widget.
        self.area.createText(title).setTitleStyle()
        #self.area.createLine()

        # Tell the wizard of us.
        wizard.addOwnedPage(self)

    def destroy(self):
        self.area.destroy()

    def getId(self):
        return self.identifier
        
    def update(self):
        pass

    def getArea(self):
        return self.area

    def setNext(self, page):
        self.next = page

    def setPrev(self, page):
        self.prev = page

    def follows(self, page):
        """This page follows another page.  Sets the next and prev
        links appropriately so that the given page is immediately
        before this one.

        @param page A WizardPage object.
        """
        self.setPrev(page)
        page.setNext(self)

    def GetNext(self):
        """Returns the next page of the wizard.  Called by
        WizardDialog."""

        return self.next

    def GetPrev(self):
        """Returns the previous page of the wizard.  Called by
        WizardDialog."""

        return self.prev



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file expressions.py Expressions
## 
## Routines for evaluating expressions.

import string
import os
import confdb as st
import paths


def evaluate(expression, contextAddon):
    """
    @param expression    Expression to evaluate as a string.

    @param contextAddon  Addon in the context of which the evaluation is made.
    
    @return  The evaluated version with all functions and variables
    expanded.
    """
    result = ''
    pos = 0

    while pos < len(expression):
        ch = expression[pos]

        # Is this the beginning of a new expression?
        if ch == '$':
            # Try to scan it.
            subExpression = scan(expression[pos:])

            if len(subExpression):
                result += execute(subExpression[1:], contextAddon)
                pos += len(subExpression)
                continue

        # Append the character to the result.
        result += ch
        pos += 1

    #print 'Evaluated result: ' + result
    return result


def scan(src):
    """Scans a subexpression that begins with $."""

    pos = 1

    # Use angle brackets < and > because they can't be used on the
    # command line.

    # $FUNCTION
    # $FUNCTION<asdas>

    while pos < len(src):
        ch = src[pos]
        if ch not in string.uppercase + '_':
            # This is no longer a part of the function name.
            if ch == '<':
                # Begin an argument block. Find the end.
                endPos = findMatchingBracket(src, pos)
                if endPos < 0:
                    # This is invalid!
                    return ''
                return src[:endPos + 1]
            break
                    
        pos += 1

    return src[:pos]


def findMatchingBracket(src, startPos):
    level = 0
    pos = startPos
    while pos < len(src):
        if src[pos] == '<':
            level += 1
        elif src[pos] == '>':
            level -= 1
        if level == 0:
            return pos
        pos += 1
    # Not good.
    return -1


def getCommand(src):
    """Extract the command from the beginning of an expression.

    @param src  Expression.  Must start at <code>$</code>.

    @return  Command as a string.
    """
    command = ''
    pos = 0
    while pos < len(src):
        if src[pos] in string.uppercase + '_':
            command += src[pos]
        else:
            break
        pos += 1
    
    return command
    

def execute(src, contextAddon):
    """Execute a command.

    @param src  Command to execute.  Must start at <code>$</code>.

    @return  The command's evaluated result.
    """

    #print "executing: " + src

    command = getCommand(src)
    if len(src) > len(command):
        argSrc = src[len(command) + 1 : -1]
    else:
        argSrc = ''

    #print 'command: ' + command + ';'
    #print 'argSrc: ' + argSrc + ';'

    if command == 'BUNDLE':
        if contextAddon:
            return contextAddon.getContentPath()
        else:
            return ''
            
    if command == 'SBROOT':
        import __main__
        arg = evaluate(argSrc, contextAddon)
        return paths.quote(os.path.join(os.path.dirname(
            os.path.abspath(__main__.__file__)), arg))

    if command == 'DENGBASE':
        arg = evaluate(argSrc, contextAddon)
        if st.isDefined('doomsday-base'):
            return paths.quote(os.path.abspath(os.path.join(
                st.getSystemString('doomsday-base'), arg)))
        else:
            return '}' + arg
        
    if command == 'PATH':
        arg = os.path.normpath(evaluate(argSrc, contextAddon))
        return paths.quote(arg)

    if command == 'VALUE':
        arg = evaluate(argSrc, contextAddon)
        import sb.profdb
        activeProfile = sb.profdb.getActive()

        # Try local value identifier first.
        if contextAddon:
            localized = contextAddon.makeLocalId(arg)
            v = activeProfile.getValue(localized)
            if v:
                return v.getValue()

        v = activeProfile.getValue(arg)
        if v:
            return v.getValue()

        return ''

    if command == 'QUOTE':
        return paths.quote( evaluate(argSrc, contextAddon) )

    return ''



# qmake_msvc.py is a Python 2 script that generates a full Visual Studio 
# solution with a .vcxproj for each subproject. You must set up envconfig.bat 
# before running the script. The solution is always placed in a folder called 
# "doomsday-msvc-build" at the repository root.
#
# qmake_msvc.py must be called whenever the .pro/.pri files change. The .sln
# and .vcxproj files are then rewritten -- manual edits will be gone!
# 
# The solution automatically deploys all built binaries into the
# "distrib\products" folder. The user is expected to 1) copy the required
# dependency .dlls to products\bin, and 2) define the appropriate launch
# command line and options (which are persistently then stored in the 
# .vcxproj.user file).

import os, sys, re

print "\nVISUAL STUDIO SOLUTION GENERATOR FOR DOOMSDAY\n"
print "Author: <jaakko.keranen@iki.fi>\n"

# Figure out the Doomsday root path.
rootPath = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), '..', '..'))
solPath = os.path.abspath(os.path.join(rootPath, '..', 'doomsday-msvc-build'))

print "Doomsday root path:", rootPath
print "Solution location: ", solPath

if not os.path.exists(solPath): os.mkdir(solPath)

# Run qmake to generate the solution and vcprojs.
os.chdir(solPath)
os.system(os.path.join(rootPath, 'build', 'win32', 'qmake_tp_vc.bat'))

# Cleanup.
print "\nDeleting spurious PK3s..."
for name in ['doomsday', 'libdoom', 'libheretic', 'libhexen', 'libdoom64']:
    fn = "..\%s.pk3" % name
    if os.path.exists(fn):
        os.remove(fn)

print "\nPatching..."

def fileRegex(fn, pattern, repl, count=0, startAtPos=0):
    text = file(fn, 'r').read()
    skipped = text[:startAtPos]
    text = text[startAtPos:]
    text = re.sub(pattern, repl, text, count)
    file(fn, 'w').write(skipped + text)
    
def fileRegOff(fn, pattern, startAtPos=0):
    text = file(fn, 'r').read()
    text = text[startAtPos:]
    found = re.search(pattern, text)
    return found.start()
    

for proj in ['libcore\deng_core.vcxproj', 'tools\shell\shell-gui\Doomsday-Shell.vcxproj']:
    fileRegex(proj, 
              r'=\\&quot;([A-Za-z0-9_\-\. ]+)\\&quot;', 
              r'=&quot;\1&quot;')
    
copyScript = 'python &quot;%s\\\\build\win32\copy_to_products.py' % rootPath + '&quot; &quot;' + solPath + '&quot;'
rcDirs = '&quot;' + rootPath + '\\\\api&quot;;&quot;' + rootPath + '\\\\liblegacy\include' + '&quot;'

def find_projs(path):
    found = []
    for name in os.listdir(path):
        fn = os.path.join(path, name)
        if os.path.isdir(fn) and '.' not in name:
            found += find_projs(fn)
        elif name[-8:] == '.vcxproj':
            found.append(os.path.join(path, name))
    return found
 
allProjs = find_projs(solPath)

print "Found %i projects." % len(allProjs)

for fn in allProjs:
    pName = os.path.basename(fn)
    pName = pName[:pName.index('.')]
    
    # Fix target names.
    fileRegex(fn, '[0-2]</PrimaryOutput>', '</PrimaryOutput>')
    fileRegex(fn, '[0-2]</TargetName>', '</TargetName>') 

    # Add a post-build step for Debug config.
    start = fileRegOff(fn, r'<ItemDefinitionGroup Condition.*Debug\|Win32')
    fileRegex(fn, 
              r'</ResourceCompile>\n',
              r"""</ResourceCompile>
    <PostBuildEvent>
      <Command>""" + copyScript + r""" Debug</Command>
    </PostBuildEvent>
    <PostBuildEvent>
      <Message>Copy """ + pName + r""" to products/bin</Message>
    </PostBuildEvent>\n""", 1, start)
    
    # Tell RC where to find headers.
    fileRegex(fn,
              r'</ResourceCompile>\n',
              r"""  <AdditionalIncludeDirectories>""" + rcDirs + """</AdditionalIncludeDirectories>\n    </ResourceCompile>\n""", 
              1, start)
   
    # Add a post-build step for Release config.
    start = fileRegOff(fn, r'<ItemDefinitionGroup Condition.*Release\|Win32')

    fileRegex(fn, 
              r'</ResourceCompile>\n',
              r"""</ResourceCompile>
    <PostBuildEvent>
      <Command>""" + copyScript + r""" Release</Command>
    </PostBuildEvent>
    <PostBuildEvent>
      <Message>Copy """ + pName + r""" to products/bin</Message>
    </PostBuildEvent>\n""", 1, start)
    
    # Tell RC where to find headers.
    fileRegex(fn,
              r'</ResourceCompile>\n',
              r"""  <AdditionalIncludeDirectories>""" + rcDirs + """</AdditionalIncludeDirectories>\n    </ResourceCompile>\n""", 
              1, start)
              
    # Enable debug symbols in all configs, not just Debug.
    fileRegex(fn, r'<GenerateDebugInformation>false</GenerateDebugInformation>', 
              '<GenerateDebugInformation>true</GenerateDebugInformation>')
    
print "\nAll done!\n"
print 'Solution generated:', os.path.join(solPath, 'doomsday.sln')



# coding=utf-8
import os
import string
import utils
from event import Event
import config

def encodedText(logText):
    logText = logText.replace('&', '&amp;')
    logText = logText.replace(u'ä', u'&auml;')
    logText = logText.replace(u'ö', u'&ouml;')
    logText = logText.replace(u'Ä', u'&Auml;')
    logText = logText.replace(u'Ö', u'&Ouml;')
    logText = logText.encode('utf-8')
    logText = logText.replace('<', '&lt;')
    logText = logText.replace('>', '&gt;')
    logText = filter(lambda c: c in string.whitespace or c > ' ', logText)
    return logText
    
    
def xmlEncodedText(logText):
    result = u''
    for c in logText:
        if c == u'<': 
            result += u'<![CDATA[<]]>'
        elif c == '>': 
            result += u'<![CDATA[>]]>'
        else:
            result += c
    return result


class Entry:
    def __init__(self):
        self.subject = ''
        self.extra = ''
        self.author = ''
        self.date = ''
        self.link = ''
        self.hash = ''
        self._message = ''
        self.tags = []
        self.guessedTags = []
        self.reverted = False
        
    def set_subject(self, subject):
        self.extra = ''
        
        # Check for a revert.
        if subject.startswith('Revert "') and subject[-1] == '"':
            self.reverted = True
            subject = subject[8:-1]
        
        # Remove tags from the subject.
        pos = subject.find(': ')
        if pos > 0:
            for tag in subject[:pos].split('|'):
                self.tags.append(tag.strip())
            subject = subject[pos + 1:].strip()
        
        if len(subject) == 0: subject = "Commit"
        
        # Check that the subject lines are not too long.
        MAX_SUBJECT = 100
        if len(utils.collated(subject)) > MAX_SUBJECT:
            # Find a suitable spot the break the subject line.
            pos = MAX_SUBJECT
            while subject[pos] not in string.whitespace: pos -= 1
            self.extra = '...' + subject[pos+1:] + ' '
            subject = subject[:pos] + '...'
        else:
            # If there is a single dot at the end of the subject, remove it.
            if subject[-1] == '.' and subject[-2] != '.':
                subject = subject[:-1]
        self.subject = encodedText(subject)
        
    def set_message(self, message):
        self._message = message.strip()
        if self.extra:
            self._message = self.extra + ' ' + self._message
        self._message = encodedText(self._message)
        
    def message(self, encodeHtml=False):
        if encodeHtml:
            return self._message.replace('\n\n', '<br/><br/>').replace('\n', ' ').strip()
        return self._message.strip()
        

class Changes:
    def __init__(self, fromTag, toTag):
        self.fromTag = fromTag
        self.toTag = toTag
        self.parse()
        
    def should_ignore(self, subject):
        if subject.startswith("Merge branch") or subject.startswith("Merge commit"):
            # Branch merges are not listed.
            return True
        return False
        
    def parse(self):
        tmpName = '__ctmp'

        format = '[[Subject]]%s[[/Subject]]' + \
                 '[[Author]]%an[[/Author]]' + \
                 '[[Date]]%ai[[/Date]]' + \
                 '[[Link]]http://github.com/skyjake/Doomsday-Engine/commit/%H[[/Link]]' + \
                 '[[Hash]]%H[[/Hash]]' + \
                 '[[Message]]%b[[/Message]]'
        os.system("git log %s..%s --format=\"%s\" >> %s" % (self.fromTag, self.toTag, format, tmpName))

        logText = unicode(file(tmpName, 'rt').read(), 'utf-8')
        
        os.remove(tmpName)        

        pos = 0
        self.entries = []
        self.debChangeEntries = []
        while True:
            entry = Entry()
            
            pos = logText.find('[[Subject]]', pos)
            if pos < 0: break # No more.
            end = logText.find('[[/Subject]]', pos)
            
            entry.set_subject(logText[pos+11:end])

            # Debian changelog just gets the subjects.
            if entry.subject not in self.debChangeEntries and not \
                self.should_ignore(entry.subject):
                self.debChangeEntries.append(entry.subject)

            # Author.
            pos = logText.find('[[Author]]', pos)
            end = logText.find('[[/Author]]', pos)
            entry.author = logText[pos+10:end]
            
            # Date.
            pos = logText.find('[[Date]]', pos)
            end = logText.find('[[/Date]]', pos)
            entry.date = logText[pos+8:end]
            
            # Link.
            pos = logText.find('[[Link]]', pos)
            end = logText.find('[[/Link]]', pos)
            entry.link = logText[pos+8:end]

            # Hash.
            pos = logText.find('[[Hash]]', pos)
            end = logText.find('[[/Hash]]', pos)
            entry.hash = logText[pos+8:end]

            # Message.
            pos = logText.find('[[Message]]', pos)
            end = logText.find('[[/Message]]', pos)
            entry.set_message(logText[pos+11:end])            
            
            if not self.should_ignore(entry.subject):
                self.entries.append(entry)

        self.deduce_tags()
        self.remove_reverts()
                
    def all_tags(self):
        # These words are always considered to be valid tags.
        tags = ['Cleanup', 'Fixed', 'Added', 'Refactor', 'Performance', 'Optimize', 'Merge branch']
        for e in self.entries:
            for t in e.tags + e.guessedTags:
                if t not in tags: 
                    tags.append(t)
        return tags

    def is_reverted(self, entry):
        for e in self.entries:
            if e.reverted and e.subject == entry.subject and e.tags == entry.tags:
                return True
        return False

    def remove_reverts(self):
        self.entries = filter(lambda e: not self.is_reverted(e), self.entries)

    def deduce_tags(self):
        # Look for known tags in untagged titles.
        allTags = self.all_tags()
        for entry in self.entries:
            if entry.tags: continue
            # This entry has no tags yet.    
            for tag in allTags:
                tag = tag.encode('utf-8')
                p = entry.subject.lower().find(tag.lower())
                if p < 0: continue
                if p == 0 or entry.subject[p - 1] not in string.ascii_letters + '-_':
                    entry.guessedTags.append(tag)
        
    def form_groups(self, allEntries):
        groups = {}
        for tag in self.all_tags():
            groups[tag] = []
            for e in allEntries:
                if tag in e.tags:
                    groups[tag].append(e)
            for e in allEntries:
                if tag in e.guessedTags:
                    groups[tag].append(e)
            
            for e in groups[tag]:
                allEntries.remove(e)
        
        groups['Miscellaneous'] = []
        for e in allEntries:
            groups['Miscellaneous'].append(e)
            
        return groups
    
    def pretty_group_list(self, tags):
        """Pretty group list for use in HTML output."""
        listed = ''
        if len(tags) > 1:
            listed = string.join(tags[:-1], ', ')
            listed += ' and ' + tags[-1]
        elif len(tags) == 1:
            listed = tags[0]
        return encodedText(listed)
            
    def generate(self, format):
        fromTag = self.fromTag
        toTag = self.toTag
        
        if format == 'html':
            out = file(Event(toTag).file_path('changes.html'), 'wt')

            MAX_COMMITS = 100
            entries = self.entries[:MAX_COMMITS]
            
            if len(self.entries) > MAX_COMMITS:
                print >> out, '<p>Showing %i of %i commits.' % (MAX_COMMITS, len(self.entries))
                print >> out, 'The <a href="%s">oldest commit</a> is dated %s.</p>' % \
                    (self.entries[-1].link, self.entries[-1].date)                
            
            # Form groups.
            groups = self.form_groups(entries)
            keys = groups.keys()
            # Sort case-insensitively by group name.
            keys.sort(cmp=lambda a, b: cmp(str(a).lower(), str(b).lower()))
            for group in keys:
                if not len(groups[group]): continue
                
                print >> out, '<h3>%s</h3>' % group                                
                print >> out, '<ul>'

                # Write a list entry for each commit.
                for entry in groups[group]:
                    otherGroups = []
                    for tag in entry.tags + entry.guessedTags:
                        if tag != group:
                            otherGroups.append(tag)
                            
                    others = self.pretty_group_list(otherGroups)
                    if others: others = ' (&rarr; %s)' % others
                    
                    print >> out, '<li>'    
                    print >> out, '<a href="%s">%s</a>: ' % (entry.link, entry.date[:10])
                    print >> out, '<b>%s</b>' % entry.subject
                    print >> out, 'by <i>%s</i>%s' % (encodedText(entry.author), others)
                    print >> out, '<blockquote style="color:#666;">%s</blockquote>' % entry.message(encodeHtml=True)
                    
                print >> out, '</ul>'
            out.close()
            
        elif format == 'xml':
            out = file(Event(toTag).file_path('changes.xml'), 'wt')
            print >> out, '<commitCount>%i</commitCount>' % len(self.entries)
            print >> out, '<commits>'
            for entry in self.entries:
                print >> out, '<commit>'
                print >> out, '<submitDate>%s</submitDate>' % entry.date
                out.write((u'<author>%s</author>\n' % xmlEncodedText(entry.author)).encode('utf-8'))
                print >> out, '<repositoryUrl>%s</repositoryUrl>' % entry.link
                print >> out, '<sha1>%s</sha1>' % entry.hash
                if entry.tags or entry.guessedTags:
                    print >> out, '<tags>'
                    for t in entry.tags:
                        print >> out, '<tag guessed="false">%s</tag>' % xmlEncodedText(t)
                    for t in entry.guessedTags:
                        print >> out, '<tag guessed="true">%s</tag>' % xmlEncodedText(t)
                    print >> out, '</tags>'
                print >> out, '<title>%s</title>' % entry.subject
                if len(entry.message()):
                    print >> out, '<message>%s</message>' % entry.message()
                print >> out, '</commit>'                
            print >> out, '</commits>'
            out.close()
            
        elif format == 'deb':
            import build_version
            build_version.find_version()

            # Append the changes to the debian package changelog.
            os.chdir(os.path.join(config.DISTRIB_DIR))
            
            # First we need to update the version.
            debVersion = build_version.DOOMSDAY_VERSION_FULL + '-' + Event().tag()

            # Always make one entry.
            print 'Marking new version...'
            msg = 'New release: %s build %i.' % (build_version.DOOMSDAY_RELEASE_TYPE,
                                                 Event().number())
            
            # Reset the changelog.
            os.system('rm -f debian/changelog && ' + \
                'dch --check-dirname-level=0 --create --package doomsday -v %s "%s"' % (debVersion, msg))

            for entry in self.debChangeEntries:
                # Quote it for the command line.
                qch = entry.replace('"', '\\"').replace('!', '\\!')
                print ' *', qch
                os.system("dch --check-dirname-level 0 -a \"%s\"" % qch)

            os.system('dch --release ""')



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file button.py Button Controls

import sys, wx
import host, events, language, paths
import sb.profdb as pr
import sb.confdb as st
from widgets import uniConv
import base


class Button (base.Widget):
    """The Button class implements a button that emits a Command event
    when pushed.  There are three styles available:
    - STYLE_NORMAL: A normal button.
    - STYLE_DEFAULT: A default button.
    - STYLE_MINI: A small button.
    """

    STYLE_NORMAL = 'normal'
    STYLE_DEFAULT = 'default'
    STYLE_MINI = 'mini'

    def __init__(self, parent, wxId, id, style):
        # Create a wxButton of the appropriate type.
        if style != Button.STYLE_MINI:
            widget = wx.Button(parent, wxId,
                               uniConv(language.translate(id)))
        else:
            widget = wx.ToggleButton(parent, wxId,
                                     uniConv(language.translate(id)))

        # Default buttons are usually a bit different visually.
        if style == Button.STYLE_DEFAULT:
            widget.SetDefault()

        base.Widget.__init__(self, widget)
        self.widgetId = id

        if style == Button.STYLE_MINI:
            # The mini buttons use the small style.
            self.setSmallStyle()
            # The size of the mini buttons is configurable.
            bestSize = widget.GetBestSize()
            width = 30
            height = bestSize[1]
            try:
                width = st.getSystemInteger('button-mini-width')
                if st.getSystemInteger('button-mini-height'):
                    height = st.getSystemInteger('button-mini-height')
            except:
                pass
            widget.SetMinSize((width, height))

        if style != Button.STYLE_MINI:
            # We will handle the click event ourselves.
            wx.EVT_BUTTON(parent, wxId, self.onClick)
            #self.updateDefaultSize()
        else:
            # Pop back up when toggled down.
            wx.EVT_TOGGLEBUTTON(parent, wxId, self.onToggle)

    def updateDefaultSize(self):
        w = self.getWxWidget()
        w.SetMinSize(w.GetBestSize())

    def retranslate(self):
        self.getWxWidget().SetLabel(uniConv(language.translate(self.widgetId)))

    def onClick(self, ev):
        """Handle a wxWidgets button click event."""
        events.send(events.Command(self.widgetId))

        # This causes a reaction.
        self.react()

    def onToggle(self, ev):
        """Handle a wxWidgets toggle button click event."""
        # Since this is supposed to be a normal button and not a
        # toggle button, return to the Off-state automatically.
        self.getWxWidget().SetValue(False)
        events.send(events.Command(self.widgetId))

        # This causes a reaction.
        self.react()


class CheckBox (base.Widget):
    """A check box widget."""

    def __init__(self, parent, wxId, label, isChecked):
        base.Widget.__init__(self, wx.CheckBox(parent, wxId,
                                               uniConv(language.translate(label)),
                                               style=wx.CHK_3STATE |
                                               wx.CHK_ALLOW_3RD_STATE_FOR_USER))
        self.widgetId = label

        # Set the initial value of the checkbox.
        w = self.getWxWidget()
        if isChecked:
            w.Set3StateValue(wx.CHK_CHECKED)
        else:
            w.Set3StateValue(wx.CHK_UNCHECKED)

        # We want focus notifications.
        self.setFocusId(self.widgetId)

        self.indicator = None

        # When the checkbox is focused, send a notification.
        wx.EVT_CHECKBOX(parent, wxId, self.onClick)
        
        # Listen for value changes.
        self.addValueChangeListener()
        self.addProfileChangeListener()

    def setDefaultIndicator(self, indicatorTextWidget):
        """Set the Text widget that displays when the check box is in
        the Default state.

        @param indicatorTextWidget  A Widgets.Text widget.
        """
        self.indicator = indicatorTextWidget
        self.indicator.getWxWidget().SetForegroundColour(
            wx.Colour(170, 170, 170))
        self.indicator.setSmallStyle()
        self.updateState()

        # Clicking on the indicator will move focus to the setting widget.
        self.indicator.setFocusId(self.widgetId)

    def updateState(self):
        """Update the state of the indicator to match one of the three
        states."""

        if self.indicator:
            w = self.getWxWidget()
            if w.Get3StateValue() == wx.CHK_UNDETERMINED:
                self.indicator.setText(
                    '(' + language.translate('toggle-use-default-value') + \
                    language.translate(
                    pr.getDefaults().getValue(self.widgetId).getValue()) + ')')
            else:
                self.indicator.setText('')

    def onClick(self, event):
        """Swap through the three states when the check box is clicked."""

        if self.widgetId:
            w = self.getWxWidget()
            state = w.Get3StateValue()

            if (state == wx.CHK_UNDETERMINED and 
                pr.getActive() is pr.getDefaults()):
                state = wx.CHK_UNCHECKED
                w.Set3StateValue(state)

            if state == wx.CHK_CHECKED:
                pr.getActive().setValue(self.widgetId, 'yes')
                newValue = 'yes'
                
            elif state == wx.CHK_UNCHECKED:
                pr.getActive().setValue(self.widgetId, 'no')
                newValue = 'no'

            else:
                pr.getActive().removeValue(self.widgetId)
                newValue = 'default'

            self.updateState()
            events.send(events.EditNotify(self.widgetId, newValue))
        
        # Let wxWidgets process the event, too.
        event.Skip()

    def getFromProfile(self, profile):
        # Get the value for the setting as it has been defined
        # in the currently active profile.
        value = profile.getValue(self.widgetId, False)
        w = self.getWxWidget()
        if value:
            if value.getValue() == 'yes':
                w.Set3StateValue(wx.CHK_CHECKED)
            else:
                w.Set3StateValue(wx.CHK_UNCHECKED)
        else:
            w.Set3StateValue(wx.CHK_UNDETERMINED)
        self.updateState()

    def onNotify(self, event):
        """Handle notifications.  When the active profile changes, the
        check box's state is updated."""
        
        base.Widget.onNotify(self, event)

        if self.widgetId:
            w = self.getWxWidget()
            if event.hasId('active-profile-changed'):
                self.getFromProfile(pr.getActive())

            elif event.hasId(self.widgetId + '-value-changed'):
                if event.getValue() == 'yes':
                    w.Set3StateValue(wx.CHK_CHECKED)
                elif event.getValue() == 'no':
                    w.Set3StateValue(wx.CHK_UNCHECKED)
                elif event.getValue() is None:
                    w.Set3StateValue(wx.CHK_UNDETERMINED)
                self.updateState()

    def retranslate(self):
        if self.widgetId:
            self.getWxWidget().SetLabel(
                uniConv(language.translate(self.widgetId)))


class RadioButton (base.Widget):
    """A radio button widget."""

    def __init__(self, parent, wxId, label, isToggled, isFirst=False):
        if isFirst:
            flags = wx.RB_GROUP
        else:
            flags = 0
        base.Widget.__init__(self, wx.RadioButton(parent, wxId, 
                                                  language.translate(label), 
                                                  style=flags))
        self.label = label
        
        if isToggled:
            self.getWxWidget().SetValue(True)
        
    def setText(self, text):
        self.getWxWidget().SetLabel(text)
        
    def isChecked(self):
        return self.getWxWidget().GetValue()



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file field.py Editor Fields

import sys
import wx
import wx.lib.intctrl as intctrl
import host, events, language, paths
import sb.profdb as pr
import base


def uniDeconv(str):
    """Encode Unicode back to local character encoding.

    @param str  Unicode string.
    
    @return String.
    """
    if host.isWindows():
        # Don't do Unicode on Windows.
        return str
    elif type(str) != unicode:
        return str
    else:
        return str.encode(host.getEncoding())


class TextField (base.Widget):
    """A text field for entering a single-line string of text."""

    def __init__(self, parent, wxId, id):
        base.Widget.__init__(self, wx.TextCtrl(parent, wxId))
        self.widgetId = id

        # The default font looks the best.
        self.getWxWidget().SetFont(wx.NORMAL_FONT)

        # We want focus notifications.
        self.setFocusId(id)

        # Listen for content changes.
        wx.EVT_TEXT(parent, wxId, self.onTextChange)

        # This must be set to True when notifications need to be sent.
        self.willNotify = True

        # If True, a value-changed notification will be immediately
        # reflected in the text field.
        self.reactToNotify = True

        # The default validator accepts anything.
        self.validator = lambda text: True
        
        # Listen to value changes.
        self.addValueChangeListener()
        self.addProfileChangeListener()

    def getText(self):
        """Return the text in the text field."""
        return uniDeconv(self.getWxWidget().GetValue())

    def setText(self, text):
        self.willNotify = False
        if text != None:
            self.getWxWidget().SetValue(text)
        else:
            self.getWxWidget().SetValue('')
        self.willNotify = True

        # This does cause a reaction, though.
        self.react()

    def select(self, fromPos=-1, toPos=-1):
        """Selects a range in the text field. By default, selects everything.
        @param fromPos  Start position for the selection.
        @param endPos  End position for the selection.
        """
        self.getWxWidget().SetSelection(fromPos, toPos)

    def setValidator(self, validatorFunc):
        """Set the validator function.  The validator returns True if
        the text given to it is valid.  By default anything is valid.

        @param validatorFunc A function that takes the text as a
        parameter.
        """
        self.validator = validatorFunc

    def onTextChange(self, ev):
        """Handle the wxWidgets event that occurs when the contents of
        the field are updated (for any reason).

        @param ev wxWidgets event.
        """
        if self.willNotify and self.widgetId:
            newText = self.getWxWidget().GetValue()

            if not self.validator(newText):
                # The new text is not valid.  Nothing will be done.
                return

            self.reactToNotify = False
            if not len(newText):
                pr.getActive().removeValue(self.widgetId)
            else:
                pr.getActive().setValue(self.widgetId, newText)
            self.reactToNotify = True

            # Send a notification.
            events.sendAfter(events.EditNotify(self.widgetId, newText))

        self.react()
        
    def getFromProfile(self, profile):
        """Gets the text from a profile. Needs a widget identifier."""
        # Get the value for the setting as it has been defined
        # in the currently active profile.
        value = profile.getValue(self.widgetId, False)
        if value:
            self.setText(value.getValue())
        else:
            self.setText('')
        
    def onNotify(self, event):
        """Notification listener.

        @param event An events.Notify object.
        """
        base.Widget.onNotify(self, event)

        if self.reactToNotify and event.hasId(self.widgetId + '-value-changed'):
            # This is our value.
            self.setText(event.getValue())

        if self.widgetId and event.hasId('active-profile-changed'):
            self.getFromProfile(pr.getActive())
        

class NumberField (base.Widget):
    def __init__(self, parent, wxId, id):
        base.Widget.__init__(self, intctrl.IntCtrl(parent, wxId,
                                                   limited=False,
                                                   allow_none=True))
        self.widgetId = id

        # Clear the widget by default.
        self.getWxWidget().SetValue(None)

        # The default font looks the best.
        self.getWxWidget().SetFont(wx.NORMAL_FONT)

        # If True, a value-changed notification will cause the field
        # to change.
        self.reactToNotify = True

        # We want focus notifications.
        self.setFocusId(id)

        # Listen for changes.
        intctrl.EVT_INT(parent, wxId, self.onChange)
        
        # Listen to value changes.
        self.addValueChangeListener()
        self.addProfileChangeListener()

    def setRange(self, min, max):
        """Set the allowed range for the field.  Setting a limit to
        None will remove that limit entirely.

        @param min Minimum value for the range.
        @param max Maximum value for the range.
        """
        self.getWxWidget().SetBounds(min, max)

    def setValue(self, value):
        """Set a new value for the number field.

        @param value The new value.  Set to None to clear the field.
        """
        # Clear the widget by default.
        self.getWxWidget().SetValue(value)

    def onChange(self, ev):
        """Handle the wxWidgets event that is sent when the contents
        of the field changes."""
        if self.widgetId:
            if self.getWxWidget().IsInBounds():
                v = ev.GetValue()
                if v is not None:
                    newValue = str(v)
                else:
                    newValue = ''
            else:
                newValue = None

            self.reactToNotify = False
            if newValue == None:
                pr.getActive().removeValue(self.widgetId)
            else:
                pr.getActive().setValue(self.widgetId, newValue)
            self.reactToNotify = True

            # Notification about the change.
            events.sendAfter(events.EditNotify(self.widgetId, newValue))

    def onNotify(self, event):
        base.Widget.onNotify(self, event)
        
        if self.widgetId:
            if self.reactToNotify and event.hasId(self.widgetId + '-value-changed'):
                # This is our value.
                if event.getValue() != None:
                    self.setValue(int(event.getValue()))
                else:
                    self.setValue(None)

            elif event.hasId('active-profile-changed'):
                # Get the value for the setting as it has been defined
                # in the currently active profile.
                value = pr.getActive().getValue(self.widgetId, False)
                if value and value.getValue() != '':
                    self.setValue(int(value.getValue()))
                else:
                    self.setValue(None)



#!/usr/bin/env python
# -*- Coding: UTF-8 -*-

# ---------------------------------------------------------------------------
# Open Asset Import Library (ASSIMP)
# ---------------------------------------------------------------------------
#
# Copyright (c) 2006-2010, ASSIMP Development Team
#
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, 
# with or without modification, are permitted provided that the following 
# conditions are met:
# 
# * Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
# 
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
# 
# * Neither the name of the ASSIMP team, nor the names of its
#   contributors may be used to endorse or promote products
#   derived from this software without specific prior
#   written permission of the ASSIMP Development Team.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ---------------------------------------------------------------------------

"""Update PyAssimp's data structures to keep up with the
C/C++ headers.

This script is meant to be executed in the source tree, directly from
port/PyAssimp/gen
"""

import os
import re

#==[regexps]=================================================

# Clean desc 
REdefine = re.compile(r''
                r'(?P<desc>)'                                       # /** *desc */
                r'#\s*define\s(?P<name>[^(\n]+?)\s(?P<code>.+)$'    #  #define name value
                , re.MULTILINE)

# Get structs
REstructs = re.compile(r''
                #r'//\s?[\-]*\s(?P<desc>.*?)\*/\s'           # /** *desc */
                #r'//\s?[\-]*(?P<desc>.*?)\*/(?:.*?)'            # garbage
                r'//\s?[\-]*\s(?P<desc>.*?)\*/\W*?'           # /** *desc */
                r'struct\s(?:ASSIMP_API\s)?(?P<name>[a-z][a-z0-9_]\w+\b)'    # struct name
                r'[^{]*?\{'                                 # {
                r'(?P<code>.*?)'                            #   code 
                r'\}\s*(PACK_STRUCT)?;'                      # };
                , re.IGNORECASE + re.DOTALL + re.MULTILINE)

# Clean desc 
REdesc = re.compile(r''
                r'^\s*?([*]|/\*\*)(?P<line>.*?)'            #  * line 
                , re.IGNORECASE + re.DOTALL + re.MULTILINE)

# Remove #ifdef __cplusplus   
RErmifdef = re.compile(r''
                r'#ifdef __cplusplus'                       # #ifdef __cplusplus
                r'(?P<code>.*)'                             #   code 
                r'#endif(\s*//\s*!?\s*__cplusplus)*'          # #endif
                , re.IGNORECASE + re.DOTALL)
                
# Replace comments
RErpcom = re.compile(r''
                r'\s*(/\*+\s|\*+/|\B\*\s|///?!?)'             # /**
                r'(?P<line>.*?)'                            #  * line 
                , re.IGNORECASE + re.DOTALL)
                
# Restructure 
def GetType(type, prefix='c_'):
    t = type
    while t.endswith('*'):
        t = t[:-1]
    if t[:5] == 'const':
        t = t[5:]

    # skip some types
    if t in skiplist:
           return None

    t = t.strip()
    types = {'unsigned int':'uint', 'unsigned char':'ubyte',}
    if t in types:
        t = types[t]
    t = prefix + t
    while type.endswith('*'):
        t = "POINTER(" + t + ")"
        type = type[:-1]
    return t
    
def restructure( match ):
    type = match.group("type")
    if match.group("struct") == "":
        type = GetType(type)
    elif match.group("struct") == "C_ENUM ":
        type = "c_uint"
    else:
        type = GetType(type[2:], '')
        if type is None:
           return ''
    if match.group("index"):
        type = type + "*" + match.group("index")
        
    result = ""
    for name in match.group("name").split(','):
        result += "(\"" + name.strip() + "\", "+ type + "),"
    
    return result

RErestruc = re.compile(r''
                r'(?P<struct>C_STRUCT\s|C_ENUM\s|)'                     #  [C_STRUCT]
                r'(?P<type>\w+\s?\w+?[*]*)\s'                           #  type
                #r'(?P<name>\w+)'                                       #  name
                r'(?P<name>\w+|[a-z0-9_, ]+)'                               #  name
                r'(:?\[(?P<index>\w+)\])?;'                             #  []; (optional)
                , re.DOTALL)
#==[template]================================================
template = """
class $NAME$(Structure):
    \"\"\"
$DESCRIPTION$
    \"\"\" 
$DEFINES$
    _fields_ = [
            $FIELDS$
        ]
"""

templateSR = """
class $NAME$(Structure):
    \"\"\"
$DESCRIPTION$
    \"\"\" 
$DEFINES$

$NAME$._fields_ = [
            $FIELDS$
        ]
"""

skiplist = ("FileIO", "File", "locateFromAssimpHeap",'LogStream','MeshAnim','AnimMesh')

#============================================================
def Structify(fileName):
    file = open(fileName, 'r')
    text = file.read()
    result = []
    
    # Get defines.
    defs = REdefine.findall(text)
    # Create defines
    defines = "\n"
    for define in defs:
        # Clean desc 
        desc = REdesc.sub('', define[0])
        # Replace comments
        desc = RErpcom.sub('#\g<line>', desc)
        defines += desc
	if len(define[2].strip()):
            # skip non-integral defines, we can support them right now
            try:
                int(define[2],0)
            except:
                continue
            defines += " "*4 + define[1] + " = " + define[2] + "\n"  
            
    
    # Get structs
    rs = REstructs.finditer(text)

    fileName = os.path.basename(fileName)
    print fileName
    for r in rs:
        name = r.group('name')[2:]
        desc = r.group('desc')
        
        # Skip some structs
        if name in skiplist:
            continue

        text = r.group('code')

        # Clean desc 
        desc = REdesc.sub('', desc)
        
        desc = "See '"+ fileName +"' for details." #TODO  
        
        # Remove #ifdef __cplusplus
        text = RErmifdef.sub('', text)
        
        # Whether the struct contains more than just POD
        primitive = text.find('C_STRUCT') == -1

        # Restructure
        text = RErestruc.sub(restructure, text)
        # Replace comments
        text = RErpcom.sub('# \g<line>', text)
        text = text.replace("),#", "),\n#")
        text = text.replace("#", "\n#")
        text = "".join([l for l in text.splitlines(True) if not l.strip().endswith("#")]) # remove empty comment lines
        
        # Whether it's selfreferencing: ex. struct Node { Node* parent; };
        selfreferencing = text.find('POINTER('+name+')') != -1
        
        complex = name == "Scene"
        
        # Create description
        description = ""
        for line in desc.split('\n'):
            description += " "*4 + line.strip() + "\n"  
        description = description.rstrip()

        # Create fields
        fields = ""
        for line in text.split('\n'):
            fields += " "*12 + line.strip() + "\n"
        fields = fields.strip()

        if selfreferencing:
            templ = templateSR
        else:
            templ = template
            
        # Put it all together
        text = templ.replace('$NAME$', name)
        text = text.replace('$DESCRIPTION$', description)
        text = text.replace('$FIELDS$', fields)
        
        if ((name.lower() == fileName.split('.')[0][2:].lower()) and (name != 'Material')) or name == "String":
            text = text.replace('$DEFINES$', defines)
        else:
            text = text.replace('$DEFINES$', '')

        
        result.append((primitive, selfreferencing, complex, text))
             
    return result   

text = "#-*- coding: UTF-8 -*-\n\n"
text += "from ctypes import POINTER, c_int, c_uint, c_size_t, c_char, c_float, Structure, c_char_p, c_double, c_ubyte\n\n"

structs1 = ""
structs2 = ""
structs3 = ""
structs4 = ""

path = '../../../include/assimp'
files = os.listdir (path)
#files = ["aiScene.h", "aiTypes.h"]
for fileName in files:
    if fileName.endswith('.h'):
        for struct in Structify(os.path.join(path, fileName)):
            primitive, sr, complex, struct = struct
            if primitive:
                structs1 += struct
            elif sr:
                structs2 += struct
            elif complex:
                structs4 += struct
            else:
                structs3 += struct

text += structs1 + structs2 + structs3 + structs4

file = open('structs.py', 'w')
file.write(text)
file.close()

print("Generation done. You can now review the file 'structs.py' and merge it.")



#!/usr/bin/env python2.7
# Determining the number of a build.

import time
import sys

def todays_build():
    now = time.localtime()
    return str((now.tm_year - 2011)*365 + now.tm_yday)
    
if '--print' in sys.argv:
    print todays_build()



#!/usr/bin/env python2.7
# coding=utf-8
#
# The Doomsday Build Pilot
# (c) 2011-2012 Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file pilot.py
##
## The build pilot runs on the autobuilder systems. On the host it manages
## tasks for the builder client systems. It listens on a TCP port for incoming
## queries and actions. On the clients the pilot is run periodically by cron,
## and any new tasks are carried out.
##
## The pilot's responsibility is distributed task management; the autobuild
## script carries out the actual tasks.

import sys
import os
import platform
import subprocess
import pickle
import struct
import time
import SocketServer
import builder.utils

def homeDir():
    """Determines the path of the pilot home directory."""
    if 'Windows' in platform.platform():
        return os.path.join(os.getenv('USERPROFILE'), '.pilot')
    return os.path.join(os.getenv('HOME'), '.pilot')

# Get the configuration.
try:
    sys.path.append(homeDir())
    import pilotcfg
except ImportError:
    print """Configuration needed: pilotcfg.py must be created in ~/.pilot/

pilotcfg.py contains information such as (global variables):
- HOST: pilot server address (for clients)
- PORT: pilot server port
- ID: identifier of the build system
- DISTRIB_DIR: distrib directory path
- EVENTS_DIR: events directory path
- APT_DIR: apt repository path (for Linux systems)
- IGNORED_TASKS: list of tasks to quietly ignore (marked as complete)

The function 'postTaskHook(task)' can be defined for actions to be carried out 
after a successful execution of a task."""
    sys.exit(1)

APP_NAME = 'Doomsday Build Pilot'

def main():
    checkHome()
    checkMasterActions()    

    startNewPilotInstance()
    try:
        if isServer():
            listen()
        else:
            # Client mode. Check quietly for new tasks.
            checkForTasks()
    except Exception, x:
        # Unexpected problem!
	import traceback
	traceback.print_exc()
        print APP_NAME + ':', x
    finally:
        endPilotInstance()

def msg(s):
    print >> sys.stderr, s


def isServer():
    return 'server' in sys.argv


def checkHome():
    if not os.path.exists(homeDir()):
        raise Exception(".pilot home directory does not exist.")


def branchFileName():
    return os.path.join(homeDir(), 'branch')


def currentBranch():
    if not os.path.exists(branchFileName()):
        return 'master' # default branch
    return file(branchFileName(), 'rt').read().strip()


def switchToBranch(branch):
    f = file(branchFileName(), 'wt')
    print >> f, branch
    f.close()


def checkMasterActions():
    """Special master actions."""
    if len(sys.argv) < 2: return
    
    if sys.argv[1] == 'new':
        assert pilotcfg.ID == 'master'
        
        # Create a new task: new sysid[,sysid]* taskname
        target = sys.argv[2]
        taskName = sys.argv[3]
        if target == 'ALL':
            newTask(taskName, allClients=True)
        else:
            for tgt in target.split(','):
                newTask(taskName, forClient=tgt)
        sys.exit(0)
        
    if sys.argv[1] == 'finish':
        assert pilotcfg.ID == 'master'
        
        # Check for completed tasks.
        handleCompletedTasks()
        sys.exit(0)    


def pidFileName():
    if isServer():
        return 'server.pid'
    else:
        return 'client.pid'


def isStale(fn):
    """Files are considered stale after some time has passed."""
    age = time.time() - os.stat(fn).st_ctime
    if age > 4*60*60:
        msg(fn + ' is stale, ignoring it.')
        return True
    return False


def startNewPilotInstance():
    """A new pilot instance can only be started if one is not already running
    on the system. If an existing instance is detected, this one will quit
    immediately."""
    # Check for an existing pid file.
    pid = os.path.join(homeDir(), pidFileName())
    if os.path.exists(pid):
        if not isStale(pid):
            # Cannot start right now -- will be retried later.
            sys.exit(0)
    print >> file(pid, 'w'), str(os.getpid())
    
    
def endPilotInstance():
    """Ends this pilot instance."""
    os.remove(os.path.join(homeDir(), pidFileName()))
    
    
def listTasks(clientId=None, includeCompleted=True, onlyCompleted=False,
              allClients=False):
    tasks = []

    for name in os.listdir(homeDir()):
        fn = os.path.join(homeDir(), name)
        if fn == '.' or fn == '..': continue

        # All tasks are specific to a client.
        if os.path.isdir(fn) and (name == clientId or allClients):
            for subname in os.listdir(fn):
                # Every task begins with the task prefix.
                if not subname.startswith('task_'): continue
                subfn = os.path.join(fn, subname)
                if not os.path.isdir(subfn):
                    tasks.append(subname[5:]) # Remove prefix.
    
    if not includeCompleted:
        tasks = filter(lambda n: not n.endswith('.done'), tasks)
        
    if onlyCompleted:
        tasks = filter(lambda n: isTaskComplete(n), tasks)

    tasks.sort()
    return tasks
            
            
def packs(s):
    """The length of the string is prefixed as a 32-bit integer in network
    byte order."""
    return struct.pack('!i', len(s)) + s

        
class ReqHandler(SocketServer.StreamRequestHandler):
    """Handler for requests from clients."""
    
    def handle(self):
        try:
            bytes = struct.unpack('!i', self.rfile.read(4))[0]
            self.request = pickle.loads(self.rfile.read(bytes))
            if type(self.request) != dict:
                raise Exception("Requests must be of type 'dict'")
            self.doRequest()
        except Exception, x:
            msg('Request failed: ' + str(x))
            response = { 'result': 'error', 'error': str(x) }
            self.respond(response)
            
    def respond(self, rsp):
        self.wfile.write(packs(pickle.dumps(rsp, 2)))
            
    def clientId(self):
        if 'id' in self.request:
            return self.request['id']
        else:
            return None

    def doRequest(self):
        if 'query' in self.request:
            self.doQuery()
        elif 'action' in self.request:
            self.doAction()
        else:
            raise Exception("Unknown request")
             
    def doQuery(self):
        qry = self.request['query']
        if qry == 'get_tasks':
            # Returns the tasks that a client should work on next.
            self.respond({ 'tasks': listTasks(self.clientId(),
                includeCompleted=False), 'result': 'ok' })
        else:
            raise Exception("Unknown query: " + qry)
              
    def doAction(self):
        act = self.request['action']
        if act == 'complete_task':
            completeTask(self.request['task'], self.clientId())
            self.respond({ 'result': 'ok', 'did_action': act })
        else:
            raise Exception("Unknown action: " + act)
    
        
def listen():
    print APP_NAME + ' starting in server mode (port %i).' % pilotcfg.PORT
    server = SocketServer.TCPServer(('0.0.0.0', pilotcfg.PORT), ReqHandler)
    server.serve_forever()


def query(q):
    """Sends a query to the server and returns the result."""
    
    import socket
    response = None
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock.connect((pilotcfg.HOST, pilotcfg.PORT))
        sock.send(packs(pickle.dumps(q, 2)))
        bytes = struct.unpack('!i', sock.recv(4))[0]
        response = pickle.loads(sock.recv(bytes))
    finally:
        sock.close()
    return response

    
def checkForTasks():
    for task in query({'id': pilotcfg.ID, 'query': 'get_tasks'})['tasks']:
        if not doTask(task): 
            # Ignore this task... (It will be done later.)
            continue 
        
        # Check for a post-task hook.
        if 'postTaskHook' in dir(pilotcfg):
            pilotcfg.postTaskHook(task)

        # No exception was thrown -- the task was successful.
        query({'action': 'complete_task', 
               'task': task, 
               'id': pilotcfg.ID})
            

def doTask(task):
    """Throws an exception if the task fails."""

    # Are we supposed to ignore this task?
    if 'IGNORED_TASKS' in dir(pilotcfg) and \
        task in pilotcfg.IGNORED_TASKS:
        return True

    if task.startswith('branch_'):
        msg("SWITCH TO BRANCH: " + task[7:])
        autobuild('pull')
        switchToBranch(task[7:])
        return autobuild('pull')

    elif task.startswith('buildfrom_'):
        msg("SWITCH TO BRANCH FOR BUILD: " + task[10:])
        autobuild('pull')
        switchToBranch(task[10:])
        return autobuild('pull')

    elif task == 'tag_build':
        msg("TAG MASTER BRANCH")
        return autobuild('create')

    elif task == 'deb_changes':
        msg("UPDATE .DEB CHANGELOG")
        return autobuild('debchanges')

    elif task == 'mac_meta':
        msg("OS X METADATA UPDATE")
        return autobuild('mac_meta')

    elif task == 'build':
        msg("BUILD RELEASE")
        return autobuild('platform_release')

    elif task == 'source':
        msg("PACKAGE SOURCE")
        return autobuild('source')

    elif task == 'sign':
        msg("SIGN PACKAGES")
        return autobuild('sign')

    elif task == 'publish':
        msg("PUBLISH")
        return autobuild("publish")

    elif task == 'apt_refresh':
        msg("APT REPOSITORY REFRESH")
        return autobuild('apt')
        
    elif task == 'update_feed':
        msg("UPDATE FEED")
        autobuild('feed')
        autobuild('xmlfeed')
        
    elif task == 'purge':
        msg("PURGE")
        return autobuild('purge')
        
    elif task == 'generate_readme':
        msg("GENERATE README")
        return autobuild('readme')
        
    elif task == 'generate_apidoc':
        msg("GENERATE API DOCUMENTATION")
        return autobuild('apidoc')
        
    elif task == 'mirror':
        msg("MIRROR")
        systemCommand('mirror-buildmaster-to-idisk.sh')

    return True
    

def handleCompletedTasks():
    """Check the completed tasks and see if we should start new tasks."""

    while True:
        tasks = listTasks(allClients=True, onlyCompleted=True)
        if len(tasks) == 0: break

        task = tasks[0][:-5] # Remove '.done'
        assert isTaskComplete(task)
        
        clearTask(task)
        
        print "Task '%s' has been completed (noticed at %s)" % (task, time.asctime())
        
        if task.startswith('buildfrom_'):
            # Commence with a build when everyone is ready.
            newTask('tag_build', forClient='master')
        
        elif task == 'tag_build':
            newTask('deb_changes', forClient='ubuntu-32bit')
            newTask('mac_meta', forClient='master')
            newTask('generate_readme', forClient='dox')
        
        elif task == 'deb_changes':
            newTask('build', allClients=True)
            newTask('generate_wiki', forClient='master')
        
        elif task == 'build':
            newTask('source', forClient='ubuntu-64bit')

        elif task == 'source':
            newTask('sign', forClient='master')

        elif task == 'sign':
            newTask('publish', forClient='master')
            newTask('apt_refresh', forClient='ubuntu-32bit')
            # After the build we can switch to the master again.
            newTask('branch_master', allClients=True)
            
        elif task == 'publish':
            newTask('update_feed', forClient='master')
            
        elif task == 'update_feed':
            newTask('mirror', forClient='master')
    
    
def autobuild(cmd):
    cmdLine = "%s %s" % (os.path.join(pilotcfg.DISTRIB_DIR, 'autobuild.py'), cmd)
    cmdLine += " --distrib %s" % pilotcfg.DISTRIB_DIR
    if 'EVENTS_DIR' in dir(pilotcfg):
        cmdLine += " --events %s" % pilotcfg.EVENTS_DIR
    if 'APT_DIR' in dir(pilotcfg):
        cmdLine += " --apt %s" % pilotcfg.APT_DIR

    cmdLine += " --branch %s" % currentBranch()

    builder.utils.run_python2(cmdLine)
    return True
        
    
def systemCommand(cmd):
    result = subprocess.call(cmd, shell=True)
    if result != 0:
        raise Exception("Error from " + cmd)
    
    
def newTask(name, forClient=None, allClients=False):
    if allClients:
        for fn in os.listdir(homeDir()):
            if os.path.isdir(os.path.join(homeDir(), fn)):
                newTask(name, fn)
        return
                
    path = os.path.join(homeDir(), forClient)
    print "New task '%s' for client '%s'" % (name, forClient)
    
    print >> file(os.path.join(path, 'task_' + name), 'wt'), time.asctime()
        
        
def completeTask(name, byClient):
    path = os.path.join(homeDir(), byClient, 'task_' + name)
    if not os.path.exists(path):
        raise Exception("Cannot complete missing task '%s' (by client '%s')" % (name, byClient))

    print "Task '%s' completed by '%s' at" % (name, byClient), time.asctime()    
    os.rename(path, path + '.done')

        
def clearTask(name, direc=None):
    """Delete all task files with this name."""
    if not direc: direc = homeDir()
    for fn in os.listdir(direc):
        p = os.path.join(direc, fn)
        if os.path.isdir(p):
            clearTask(name, p)            
        if fn.startswith('task_') and (fn[5:] == name or \
            fn[5:] == name + '.done'):
            os.remove(p)


def isTaskComplete(name):
    # Remote the possible '.done' suffix.
    if name[-5:] == '.done': name = name[:-5]
    # Check that everyone has completed it.
    for task in listTasks(allClients=True):
        if task.startswith(name) and not task.endswith('.done'):
            # This one is not complete.
            return False
    return True
               

if __name__ == '__main__':
    main()



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2006
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file tree.py Tree Widget

import wx
import host, events, language, paths
import sb.aodb as ao
import sb.profdb as pr
import base
from widgets import uniConv


class Tree (base.Widget):
    """The tree widget. Item texts are translated from language string identifiers."""
    
    def __init__(self, parent, wxId, id):
        base.Widget.__init__(self, wx.TreeCtrl(parent, wxId))

        # Will be used when sending notifications.
        self.widgetId = id
        self.rootId = id + '-root'

        self.clear()

        # Load the tree icons.
        #images = wx.ImageList(16, 16)
        #images.Add(wx.Bitmap(paths.findBitmap('unchecked')))
        #images.Add(wx.Bitmap(paths.findBitmap('checked')))
        #images.Add(wx.Bitmap(paths.findBitmap('defcheck')))
        #if host.isWindows():
        #    images.Add(wx.Bitmap(paths.findBitmap('closedfolder')))
        #    images.Add(wx.Bitmap(paths.findBitmap('openfolder')))
        #self.getWxWidget().AssignImageList(images)

        ## @todo Should this be LEFT_UP?
        #wx.EVT_LEFT_DOWN(self.getWxWidget(), self.onItemClick)

        # Right-clicking in the tree must change the selection in
        # addition to displaying the context menu.
        #wx.EVT_RIGHT_DOWN(self.getWxWidget(), self.onRightClick)

        # We will not allow expanding totally empty categories.
        #wx.EVT_TREE_ITEM_EXPANDING(parent, wxId, self.onItemExpanding)

        wx.EVT_TREE_SEL_CHANGED(parent, wxId, self.onSelectionChanged)

        #self.grayColour = wx.Colour(192, 192, 192)

        # The root items.
        #self.removeAll()

        # Clear the list of item ancestors (addon name -> [parent items]).
        #self.itemAncestors = {}
        
        #self.listenToNotifications = True
        
        # In addition to the normal notifications, listen to addon attachment
        # and detachment notifications.
        #events.addNotifyListener(self.onNotify, ['addon-attached', 
        #                                         'addon-detached'])

    def clear(self):
        """Delete all the items in the tree."""
        self.getWxWidget().DeleteAllItems()      
        self.items = {}
        self.addItem(self.rootId)
        
    def addItem(self, identifier, parentId=None):
        """Add an item into the tree.
        
        @param identifier  Identifier of the item.
        @param parentIdentifier  Identifier of the parent item. 
                                 If None, the added item is a root-level item.
        """
        # If no parent has been specified, use the root.
        if parentId is None:
            parentId = self.rootId
            
        # Look up the parent wxTreeItemId.
        w = self.getWxWidget()
        try:
            parentItem = self.items[parentId]
            item = w.AppendItem(parentItem, language.translate(identifier))
        except:
            item = w.AddRoot(language.translate(identifier))

        # Store the identifier as the item data.
        #w.SetPyData(item, identifier)
        
        self.items[identifier] = item
                
    #def onSize(self, event):
    #    w, h = self.getWxWidget().GetClientSizeTuple()
    #    self.list.SetDimensions(0, 0, w, h)        
                
    #def insertSorted(self, parentItem, label):
    #    """Insert a tree item under the parent item, so that it's in
    #    the proper alphabetical order.
    #    """
    #    tree = self.getWxWidget()
    #    index = 0

    #    # Find the item before which the new item will be inserted.
    #    child, cookie = tree.GetFirstChild(parentItem)
    #    while child:
    #        # If we hit a Category item, we must stop.
    #        if self.getItemData(child) == None:
    #            break
    #        if cmp(label.lower(), tree.GetItemText(child).lower()) < 0:
    #            # Insert it before this one.
    #            break
    #        child = tree.GetNextSibling(child)
    #        index += 1
    #        
    #    item = tree.InsertItemBefore(parentItem, index, label)
    #    return item

    #def setItemData(self, item, data):
    #    self.getWxWidget().SetPyData(item, data)

    #def getItemData(self, item):
    #    try:
    #        data = self.getWxWidget().GetPyData(item)
    #        return data
    #    except:
    #        return None

    def retranslate(self):
        """Update all the items in the tree."""

        if pr.getActive():
            pass

    def lookupItem(self, wxItemId):
        for i in self.items.keys():
            if self.items[i] == wxItemId:
                return i
        return ''

    def getSelectedItem(self):
        w = self.getWxWidget()
        #id = w.GetPyData(w.GetSelection())
        id = self.lookupItem(w.GetSelection())
        return id
    
    def onSelectionChanged(self, treeEvent):
        """Send a notification that the selection has changed in the tree."""
        
        item = treeEvent.GetItem()
        events.send(events.SelectNotify(self.widgetId, self.lookupItem(item)))

    def onItemClick(self, ev):
        """Handle the wxWidgets event when the user clicks on the tree
        widget.

        @param ev A wxWidgets event object.
        """
        tree = self.getWxWidget()

    def selectItem(self, identifier):
        """Select a particular item in the tree.

        @param identifier  The tree item to select.
        """
        try:
            tree = self.getWxWidget()
            tree.SelectItem(self.items[identifier])
        except:
            # Ignore unknown identifiers.
            pass
        
    def getExpandedItems(self):
        """Returns a list of the identifiers of all expanded items."""
        tree = self.getWxWidget()
        expanded = []
        for identifier in self.items:
            if tree.IsExpanded(self.items[identifier]):
                expanded.append(identifier)
        return expanded
        
    def expandItem(self, identifier, doExpand=True):
        tree = self.getWxWidget()
        if doExpand:
            tree.Expand(self.items[identifier])
        else:
            tree.Collapse(self.items[identifier])

    def collapseItem(self, identifier):
        self.expandItem(identifier, False)

    def sortItemChildren(self, identifier):
        self.getWxWidget().SortChildren(self.items[identifier])



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file plugins.py Plugin Loader
##
## If a plugin init() raises an exception, the plugin is ignored.

import sys, os, re, string
import sb.confdb as st
import language, paths, logger

PLUGIN_PATH = 'plugins'

# An array of Python statements that will initialize all loaded
# plugins.
initCommands = []


def sortKey(name):
    return paths.getBase(name).lower()


def loadAll():
    "Load all the plugins from the plugins directory."

    global initCommands
    initCommands = []

    # We'll only use the modified sys.path during this function.
    oldPath = sys.path

    # Make the plugins directory the one where to look first.
    sys.path = [PLUGIN_PATH] + sys.path

    found = []

    for fileName in paths.listFiles(paths.PLUGINS):
        if paths.hasExtension('py', fileName) or \
           (paths.hasExtension('plugin', fileName) and os.path.isdir(fileName)):
            if fileName not in found:
                found.append(fileName)
                
    # Sort alphabetically across all located plugins.
    found.sort(key=sortKey)
    for fileName in found:
        if paths.hasExtension('py', fileName):
            loadSingle(fileName) # Single Python module.
        elif paths.hasExtension('plugin', fileName):            
            loadBundle(fileName) # A plugin bundle.

    # Import all plugin modules.
    for importStatement, initStatement, searchPath in initCommands:
        sys.path = searchPath
        try:
            exec importStatement
        except:
            logger.add(logger.HIGH, 'error-plugin-init-failed', importStatement,
                       logger.formatTraceback())

    # Initialize all plugins.
    for importStatement, initStatement, searchPath in initCommands:
        sys.path = searchPath
        try:
            exec initStatement
        except AttributeError, ex:
            if "'init'" not in str(ex):
                logger.add(logger.HIGH, 'error-plugin-init-failed', initStatement,
                           logger.formatTraceback())
        except:
            # Any problems during init cause the plugin to be ignored.
            logger.add(logger.HIGH, 'error-plugin-init-failed', initStatement,
                       logger.formatTraceback())
    
    sys.path = oldPath


def loadSingle(fileName, package=None):
    """Load a plugin that consists of a single .py module in a plugins
    directory.

    @param fileName File name of the the plugin.
    """
    name = paths.getBase(fileName)

    obsoletePluginNames = ['tab1_summary', 'tab2_addons', 'tab3_settings', 'tab30_addons']
    
    # Old plugin names.
    if name in obsoletePluginNames:
        logger.add(logger.MEDIUM, 'warning-obsolete-plugin', name, fileName)
    
    sysPath = sys.path
    if os.path.dirname(fileName) not in sysPath:
        sysPath = [os.path.dirname(fileName)] + sys.path   
        
    if package:
        prefix = package + '.'
    else:
        prefix = ''
   
    # Initialization will be done after everything has been loaded.
    initCommands.append(('import ' + prefix + name, prefix + name + '.init()', sysPath))


def loadBundle(path):
    """Load a plugin bundle that may contain multiple .py modules,
    plus directories.  The bundle directories will be included in file
    searching in paths.py.

    @param path The location of the plugin bundle.
    """
    contentsPath = os.path.join(path, 'Contents')

    # Include the bundle contents in the module search path (for the
    # duration of this function).
    oldSysPath = sys.path
    sys.path = [contentsPath] + sys.path
    
    # Register the bundle path into paths.py.
    paths.addBundlePath(contentsPath)

    # Make the appropriate calls to read the configuration and
    # language files in the bundle.
    readBundleConfig(contentsPath)

    # Load all of the .py modules in the bundle's Python package.
    package = paths.getBase(path)
    packagePath = os.path.join(contentsPath, package)
    for name in os.listdir(packagePath):
        fullName = os.path.join(packagePath, name)
        if name != '__init__.py' and paths.hasExtension('py', fullName):
            loadSingle(fullName, package)

    # Restore the previous module search path.
    sys.path = oldSysPath


def readBundleConfig(path):
    st.readConfigPath(os.path.join(path, 'conf'))
    language.loadPath(os.path.join(path, 'lang'))



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file template.py Template Module
##
## This module is an empty template that can be used as the basis for
## creating new modules.  You can also see examples of Doxygen
## documentation comments, such as this one.



#!/usr/bin/env python3
# -*- Coding: UTF-8 -*-

# ---------------------------------------------------------------------------
# Open Asset Import Library (ASSIMP)
# ---------------------------------------------------------------------------
#
# Copyright (c) 2006-2010, ASSIMP Development Team
#
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, 
# with or without modification, are permitted provided that the following 
# conditions are met:
# 
# * Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
# 
# * Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
# 
# * Neither the name of the ASSIMP team, nor the names of its
#   contributors may be used to endorse or promote products
#   derived from this software without specific prior
#   written permission of the ASSIMP Development Team.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# ---------------------------------------------------------------------------

"""
Run the regression test suite using the settings from settings.py.

"""

import sys
import os
import subprocess
import zipfile
import collections

import settings
import utils

# -------------------------------------------------------------------------------
EXPECTED_FAILURE_NOT_MET, DATABASE_LENGTH_MISMATCH, \
DATABASE_VALUE_MISMATCH, IMPORT_FAILURE, \
FILE_NOT_READABLE, COMPARE_SUCCESS = range(6)

messages = collections.defaultdict(lambda: "<unknown", {
        EXPECTED_FAILURE_NOT_MET:
"""Unexpected success during import\n\
\tReturn code was 0""",

        DATABASE_LENGTH_MISMATCH:
"""Database mismatch: lengths don't match\n\
\tExpected: {0} Actual: {1}""",

        DATABASE_VALUE_MISMATCH:
"""Database mismatch: """,

        IMPORT_FAILURE:
"""Unexpected failure during import\n\
\tReturn code was {0}""",

        FILE_NOT_READABLE:
"""Unexpected failure reading file""",

        COMPARE_SUCCESS:
"""Results match archived reference dump in database\n\
\tNumber of bytes compared: {0}"""
})

outfilename_output = "run_regression_suite_output.txt"
outfilename_failur = "run_regression_suite_failures.csv"
# -------------------------------------------------------------------------------
class results:
    
    """ Handle formatting of results"""

    def __init__(self, zipin):
        """Init, given a ZIPed database """
        self.failures = []
        self.success = []
        self.zipin = zipin
    
    
    def fail(self, failfile, filename_expect, pp, msg, *args):
        """
        Report failure of a sub-test
        
        File f failed a test for pp config pp, failure notice is msg, 
        *args is format()ting args for msg
        
        """
        print("[FAILURE] " + messages[msg].format(*args))
        self.failures.append((failfile, filename_expect, pp))


    def ok(self, f, pp, msg, *args):
        """
        Report success of a sub-test

        File f passed the test, msg is a happy success note, 
        *args is format()ing args for msg.

        """
        print("[SUCCESS] " + messages[msg].format(*args)) 
        self.success.append(f)


    def report_results(self):
        """Write results to ../results/run_regression_suite_failures.txt"""

        print("\n" + ('='*60) + "\n" + "SUCCESS: {0}\nFAILURE: {1}\nPercentage good: {2}".format(
            len(self.success), len(self.failures), len(self.success)/(len(self.success)+len(self.failures))  ) + 
              "\n" + ('='*60) + "\n")

        with open(os.path.join('..', 'results',outfilename_failur), "wt") as f:
            f.write("ORIGINAL FILE;EXPECTED DUMP\n")
            f.writelines(map(
                lambda x: x[0] + ' ' + x[2] + ";" + x[1] + "\n", self.failures))
        
        if self.failures:
           print("\nSee " + settings.results + "\\" + outfilename_failur 
                 + " for more details\n\n") 
    
# -------------------------------------------------------------------------------
def mkoutputdir_andgetpath(fullpath, myhash, app):
    outfile = os.path.join(settings.results, "tmp", os.path.split(fullpath)[1] + "_" + myhash)
    try:
        os.mkdir(outfile)
    except OSError:
        pass
    
    outfile = os.path.join(outfile, app)
    return outfile


# -------------------------------------------------------------------------------
def process_dir(d, outfile_results, zipin, result):
    shellparams = {'stdout':outfile_results, 'stderr':outfile_results, 'shell':False}

    print("Processing directory " + d)
    for f in os.listdir(d):
        fullpath = os.path.join(d, f)
        if os.path.isdir(fullpath) and not f == ".svn":
            process_dir(fullpath, outfile_results, zipin, result)
            continue

        for pppreset in settings.pp_configs_to_test:
            filehash = utils.hashing(fullpath, pppreset)
            failure = False
            try:
                input_expected = zipin.open(filehash, "r").read()
                # empty dump files indicate 'expected import failure'
                if not len(input_expected):
                   failure = True
            except KeyError:
                #print("Didn't find "+fullpath+" (Hash is "+filehash+") in database")
                continue

            # Ignore extensions via settings.py configured list
            # todo: Fix for multi dot extensions like .skeleton.xml
            ext = os.path.splitext(fullpath)[1].lower()
            if ext != "" and ext in settings.exclude_extensions:
                continue

            print("-"*60 + "\n  " + os.path.realpath(fullpath) + " pp: " + pppreset) 
            
            outfile_actual = mkoutputdir_andgetpath(fullpath, filehash, "ACTUAL")
            outfile_expect = mkoutputdir_andgetpath(fullpath, filehash, "EXPECT")
            outfile_results.write("assimp dump    "+"-"*80+"\n")
            outfile_results.flush()

            command = [utils.assimp_bin_path,"dump",fullpath,outfile_actual,"-b","-s","-l"]+pppreset.split()
            r = subprocess.call(command, **shellparams)

            if r and not failure:
                result.fail(fullpath, outfile_expect, pppreset, IMPORT_FAILURE, r)
                continue
            elif failure and not r:
                result.fail(fullpath, outfile_expect, pppreset, EXPECTED_FAILURE_NOT_MET)
                continue
            
            with open(outfile_expect, "wb") as s:
                s.write(input_expected) 

            try:    
                with open(outfile_actual, "rb") as s:
                    input_actual = s.read() 
            except IOError:
                continue
                
            if len(input_expected) != len(input_actual):
                result.fail(fullpath, outfile_expect, pppreset, DATABASE_LENGTH_MISMATCH,
                        len(input_expected), len(input_actual))
                continue

            outfile_results.write("assimp cmpdump "+"-"*80+"\n")
            outfile_results.flush()

            command = [utils.assimp_bin_path,'cmpdump',outfile_actual,outfile_expect]
            if subprocess.call(command, **shellparams) != 0:
                result.fail(fullpath, outfile_expect, pppreset, DATABASE_VALUE_MISMATCH)
                continue 
            
            result.ok(fullpath, pppreset, COMPARE_SUCCESS, 
                len(input_expected))     

# -------------------------------------------------------------------------------
def del_folder_with_contents(folder):
    for root, dirs, files in os.walk(folder, topdown=False):
        for name in files:
            os.remove(os.path.join(root, name))
        for name in dirs:
            os.rmdir(os.path.join(root, name))
    

# -------------------------------------------------------------------------------
def run_test():
    utils.find_assimp_or_die()
    tmp_target_path = os.path.join(settings.results, "tmp")
    try: 
        os.mkdir(tmp_target_path) 
    except OSError as oerr:
        # clear contents if tmp folder exists already
       del_folder_with_contents(tmp_target_path)

    try:
        zipin = zipfile.ZipFile(settings.database_name + ".zip",
            "r", zipfile.ZIP_STORED)
    
    except IOError:
        print("Regression database ", settings.database_name,
              ".zip was not found")
        return

    res = results(zipin)
    with open(os.path.join(settings.results, outfilename_output), "wt") as outfile:
        for tp in settings.model_directories:
            process_dir(tp, outfile, zipin, res)

    res.report_results()

# -------------------------------------------------------------------------------
if __name__ == "__main__":
    run_test()
    input("Press any key to continue ...")

# vim: ai ts=4 sts=4 et sw=4



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file observer.py Event Observer Debugging Plugin
##
## This module prints all the notify and command events to stdout.

import events


def init():
    events.addNotifyListener(handleNotify)
    events.addCommandListener(handleCommand)


def handleNotify(event):
    print
    print "Notify: " + event.getId()

    if 'selection' in dir(event):
        print "  Selection = " + str(event.getSelection())

    if 'deselection' in dir(event):
        print "  Deselection = " + str(event.getDeselection())

    if event.hasId('focus-changed'):
        print "  New focus on " + event.getFocus()

    if event.hasId('value-changed'):
        print "  New value for " + event.getSetting() + " is " + \
              str(event.getValue())

    if event.hasId('profile-updated'):
        print "  Profile =", event.getProfile().getId()
        print "  Hidden =", event.getProfile().isHidden()

    if event.hasId('widget-edited'):
        print "  New value for " + event.getWidget() + " is " + \
              str(event.getValue())

    if event.hasId('populating-area'):
        print "  Area id =", event.getAreaId()


def handleCommand(event):
    print
    print "Command: " + event.getId()





# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file tab10_summary.py Summary Tab
##
## This plugin implements the Summary tab.  It displays a basic
## summarization of the settings in the active profile.
##
## This is the default tab.  It is displayed when Snowberry is
## started.

import string
import ui, events
import sb.profdb as pr
import sb.confdb as st
import sb.aodb as ao
import sb.widget.text as wt
import language
from ui import ALIGN_HORIZONTAL

SUMMARY = 'tab-summary'

# If true, the summary tab won't be updated on notifications.
summaryDisabled = False


def init():
    "Create the Summary page."
    area = ui.createTab(SUMMARY)

    # Partition the area into various info fields.
    #bottomHalf = area.createArea(border = 4)
    #topHalf = area.createArea(border = 4)
    
    # The profile title and the description.
    #area.setWeight(1)
    #global description
    #description = area.createFormattedText()

    # The bottom half contains a summary of the active profile's
    # settings.
    area.setBorder(6)
    area.setWeight(0)

    global titleLabel
    area.setBorder(16, ui.BORDER_NOT_BOTTOM)
    titleLabel = area.createText('')
    titleLabel.setTitleStyle()
    
    area.setBorder(2, ui.BORDER_ALL)
    area.setExpanding(False)
    area.addSpacer()
    area.setExpanding(True)

    area.setBorder(16, ui.BORDER_NOT_TOP)
    area.createLine()

    area.setBorder(8, ui.BORDER_ALL)
    addonInfo = area.createArea(alignment=ALIGN_HORIZONTAL)
    area.setWeight(0)
    #systemInfo = area.createArea(alignment=ALIGN_HORIZONTAL)
    valuesInfo = area.createArea(alignment=ALIGN_HORIZONTAL)

    # Information about addons selected into the profile.
    addonInfo.setBorder(4, ui.BORDER_LEFT_RIGHT)
    addonInfo.setWeight(1)
    addonInfo.createText('loaded-addons', ':', align=wt.Text.RIGHT).setBoldStyle()
    addonInfo.setWeight(2)

    global addonListing
    addonListing = addonInfo.createText()

    # Values for the most important system settings.
    #systemInfo.setBorder(4, ui.BORDER_LEFT_RIGHT)
    #systemInfo.setWeight(1)
    #systemInfo.createText('system-settings', ':', align=wt.Text.RIGHT).setBoldStyle()
    #systemInfo.setWeight(2)

    #global systemSummary
    #systemSummary = systemInfo.createText()
    #systemSummary.setText('800x600; run in window')

    # Information about settings that have a value in the profile.
    valuesInfo.setBorder(4, ui.BORDER_LEFT_RIGHT)
    valuesInfo.setWeight(1)
    valuesInfo.createText('custom-settings', ':', align=wt.Text.RIGHT).setBoldStyle()
    valuesInfo.setWeight(2)

    global valuesListing
    valuesListing = valuesInfo.createText()

    # Listen for active profile changes.
    events.addNotifyListener(notifyHandler, ['active-profile-changed',
                                             'value-changed',
                                             'addon-attached',
                                             'addon-detached'])
    events.addCommandListener(commandHandler, ['freeze', 'unfreeze'])


def commandHandler(event):
    global summaryDisabled
    
    if event.hasId('freeze'):
        summaryDisabled = True

    elif event.hasId('unfreeze'):
        summaryDisabled = False


def notifyHandler(event):
    if summaryDisabled:
        return
    
    if event.hasId('active-profile-changed'):
        p = pr.getActive()
        titleLabel.setText(p.getName())

        # Update the summary entries.
        updateSummary(p)

        # Change to the Summary tab automatically.
        if event.hasId('active-profile-changed') and \
               st.getSystemBoolean('summary-profile-change-autoselect'):
            if event.wasChanged(): 
                # The profile did actually change.
                ui.selectTab(SUMMARY)

    elif event.hasId('value-changed') or \
         event.hasId('addon-attached') or \
         event.hasId('addon-detached'):
        # Resummarize due to a changed value of a setting.
        updateSummary(pr.getActive())


def updateSummary(profile):
    """Update the fields on the summary tab.  Each tab summarises
    certain aspects of the profile's settings.

    @param profile A profiles.Profile object.  The values will be
    taken from this profile.
    """
    # Addon listing.
    summary = []
    usedAddons = profile.getUsedAddons()
    ao.sortIdentifiersByName(usedAddons)

    usedAddons = filter(lambda a: ao.get(a).getBox() == None, usedAddons)

    for addon in usedAddons:
        # Don't let the list get too long; there is no scrolling, the
        # extra text will just get clipped...
        if len(summary) < 8:
            summary.append(language.translate(addon))

    if len(summary) == 0:
        summary = ['-']

    if len(summary) < len(usedAddons):
        # Indicate that all are not shown.
        summary[-1] += ' ...'
        
    addonListing.setText(string.join(summary, "\n"))
    addonListing.resizeToBestSize()

    # Values defined in the profile.
    summary = []

    # These are displayed in another summary field or shouldn't be
    # shown at all.
    ignoredValues = [] #'window-size', 'window-width', 'window-height', 'color-depth', 'run-in-window']

    for value in profile.getAllValues():      
        # Don't let the list get too long; there is no scrolling, the
        # extra text will just get clipped...
        if len(summary) >= 10:
            summary.append('...')
            break

        # Many settings are ignored in the summary.
        try:
            setting = st.getSetting(value.getId())
            if (setting.getGroup() == 'general-options' or
                (setting.getGroup() == 'game-options' and 'server' not in setting.getId())):
                continue
        except KeyError:
            # This isn't even a valid setting!
            continue

        if value.getId() in ignoredValues:
            continue
        
        msg = language.translate(value.getId())
        if language.isDefined(value.getValue()):
            if value.getValue() != 'yes':
                msg += ' = ' + language.translate(value.getValue()) 
        else:
            msg += ' = ' + value.getValue() 
        summary.append(msg)

    if len(summary) == 0:
        summary = ['-']
        
    valuesListing.setText(string.join(summary, "\n"))
    valuesListing.resizeToBestSize()
    
    # The system summary shows the basic display settings.
    summary = []

    # Begin with the resolution.
    #value = profile.getValue('window-size')
    #if value and value.getValue() != 'window-size-custom':
    #    summary.append(language.translate(value.getValue()))
    #else:
    #    w = profile.getValue('window-width')
    #    h = profile.getValue('window-height')
    #    if w and h:
    #        summary.append(w.getValue() + ' x ' + h.getValue())

    # Windowed mode is a special case.
    #value = profile.getValue('run-in-window')
    #if value and value.getValue() == 'yes':
    #    summary.append(language.translate('summary-run-in-window'))
    #else:
    #    value = profile.getValue('color-depth')
    #    if value:
    #        summary.append(language.translate('summary-' + \
    #                                          value.getValue()))

    #systemSummary.setText(string.join(summary, '\n'))
    #systemSummary.resizeToBestSize()

    ui.getArea(SUMMARY).updateLayout()



# -*- coding: iso-8859-1 -*-
# $Id: main.py 4303 2007-01-01 19:06:40Z skyjake $
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file installer.py Addon Installer

import os, time, string
import paths, events, ui, language
import sb.util.dialog
import sb.widget.button as wg
import sb.widget.list
import sb.profdb as pr
import sb.confdb as st
import sb.aodb as ao
import sb.addon
import logger


def run():
    # Open the file selection dialog.
    for selection in chooseAddons('install-addon-dialog',
                                  language.translate('install-addon-title'),
                                  'install'):
        try:
            ao.install(selection)
        except Exception, ex:
            logger.add(logger.HIGH, 'error-addon-installation-failed',
                       selection, str(ex))

    logger.show()


def chooseAddons(dialogId, title, actionButton):
    """Opens an addon selection dialog.

    @param title Title of the dialog.

    @param actionButton The button that will perform the affirmative
    action of the dialog.
    """
    dialog, area = sb.util.dialog.createButtonDialog(dialogId, 
        ['cancel', actionButton], 
        actionButton)

    dialog.addEndCommand('addon-dialog-add-to-custom-folders')

    # The action button is disabled until a valid selection has been
    # made.
    dialog.disableWidget(actionButton)

    area.setWeight(0)
    folderArea = area.createArea(alignment=ui.ALIGN_HORIZONTAL)
    folderArea.setExpanding(False)
    folderArea.setBorder(2, ui.BORDER_NOT_LEFT)
    folderArea.setWeight(0)
    folderArea.createText('addon-dialog-folder').resizeToBestSize()
    folderArea.setBorderDirs(ui.BORDER_ALL)
    folderArea.setWeight(1)
    pathField = folderArea.createTextField('')
    pathField.setText(os.getcwd())
    pathField.select()
    pathField.focus()
    folderArea.setWeight(0)
    folderArea.setBorderDirs(ui.BORDER_NOT_RIGHT)
    browseButton = folderArea.createButton('addon-dialog-folder-browse',
                                           style=wg.Button.STYLE_MINI)
    #folderArea.addSpacer()
    #folderArea.setBorderDirs(ui.BORDER_NOT_RIGHT)
    
    folderCmdArea = area.createArea(alignment=ui.ALIGN_HORIZONTAL)
    folderCmdArea.setExpanding(False)
    folderCmdArea.setBorder(6, ui.BORDER_NOT_TOP)
    folderCmdArea.setWeight(1)
    folderCmdArea.addSpacer()
    folderCmdArea.setWeight(0)
        
    uninstButton = folderCmdArea.createButton('addon-dialog-folder-uninstalled')
   
    # Add to Custom Folders button.
    folderCmdArea.setBorder(6, ui.BORDER_LEFT | ui.BORDER_BOTTOM)
    addToMyButton = folderCmdArea.createButton('addon-dialog-add-to-custom-folders')

    def goToUninstalled():
        pathField.setText(paths.getUserPath(paths.UNINSTALLED))
        pathField.select()
        addToMyButton.disable()

    uninstButton.addReaction(goToUninstalled)

    area.createText('addon-dialog-found')
    area.setWeight(1)
    foundList = area.createList('', style=sb.widget.list.List.STYLE_COLUMNS)
    foundList.setMinSize(500, 300)

    area.setWeight(0)
    area.createText('addon-dialog-addons-copied', maxLineLength=70).resizeToBestSize()

    def selectAction():
        dialog.enableWidget(actionButton)

    foundList.addReaction(selectAction)

    for col, width in [('name', None), ('type', 180)]:
        foundList.addColumn('addon-dialog-' + col, width)

    def updateList():
        # Update the found addons list.
        foundList.clear()
        dialog.disableWidget(actionButton)
        extensions = ao.getAddonExtensions() + ['manifest']

        # This should be done in addons.py.
        fileNames = os.listdir(pathField.getText())
        for name in fileNames:
            type = ''
            for ext in extensions:
                if paths.hasExtension(ext, name):
                    type = ext
                    break

            if not type:
                # Unknown files are skipped.
                continue

            # Manifests don't appear in the list if the corresponding
            # addon is in the same directory.
            if paths.hasExtension('manifest', name):
                foundSame = False

                # Identifier of the addon the manifest belongs to.
                manifestId = paths.getBase(name)

                # See if the addon is in the list.
                for other in fileNames:
                    if other == name:
                        continue
                    if manifestId == ao.formIdentifier(other)[0]:
                        foundSame = True
                        break                    
                if foundSame:
                    # Don't add it.
                    continue
            
            foundList.addItemWithColumns(
                name, name, language.translate('addon-dialog-type-' + type))

    # Update reactions.
    def pathChanged():
        if os.path.exists(pathField.getText()):
            updateList()
            if pathField.getText() != paths.getUserPath(paths.UNINSTALLED):
                addToMyButton.enable()
        else:
            addToMyButton.disable()

    def browseAction():
        # Show a directory browser.
        selection = sb.util.dialog.chooseFolder('addon-dialog-browse-prompt',
                                                pathField.getText())
        if len(selection):
            pathField.setText(selection)
            pathField.select()

    # The initial contents of the list.
    updateList()

    pathField.addReaction(pathChanged)
    browseButton.addReaction(browseAction)

    dialog.addEndCommand(actionButton)
    result = dialog.run()
    if result == actionButton:
        addonFiles = map(lambda name: os.path.join(pathField.getText(), name),
                         foundList.getSelectedItems())

        # Include any associated manifests.
        for name in addonFiles:
            manifest = ao.formIdentifier(name)[0] + '.manifest'
            manifestFile = os.path.join(pathField.getText(), manifest)
            if manifest not in addonFiles and os.path.exists(manifestFile):
                addonFiles.append(manifestFile)
                #print 'including ' + manifestFile + ' due to ' + name

        return addonFiles

    elif result == 'addon-dialog-add-to-custom-folders':
        paths.addAddonPath(pathField.getText())
        events.send(events.Notify('addon-paths-changed'))
        ao.refresh()
        return []

    # The dialog was canceled.
    return []



# -*- coding: iso-8859-1 -*-
# $Id$
# Snowberry: Extensible Launcher for the Doomsday Engine
#
# Copyright (C) 2004, 2005
#   Jaakko Keränen <jaakko.keranen@iki.fi>
#   Antti Kopponen <antti.kopponen@tut.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not: http://www.opensource.org/

## @file events.py Event Bus

# The event listeners. The dictionaries match event identifiers to lists
# of callbacks. The None identifier applies to all events sent.
commandListeners = {None:[]}
notifyListeners = {None:[]}

# Send counter.  Used to determine if a send() is currently underway.
sendDepth = 0

# Events waiting to be sent (FIFO queue).
queuedEvents = []

# While muted, the sending of events is prevented. All events 
# get discarded.
areEventsMuted = False


class Event:
    def __init__(self, id):
        self.id = id

    def getId(self):
        return self.id

    def hasId(self, id):
        """Test whether the event is of certain type.

        @param id The event identifier to test.

        @return True, if the event's identifier matches the specified
        id.  Otherwise false.
        """        
        return self.id == id
    
        
class Command (Event):
    def __init__(self, id):
        Event.__init__(self, id)
        self.myClass = Command


class Notify (Event):
    def __init__(self, id):
        Event.__init__(self, id)
        self.myClass = Notify


class PopulateNotify (Notify):
    """A PopulateNotify requests any listeners to populate an area
    with widgets."""

    def __init__(self, identifier, area):
        Notify.__init__(self, 'populating-area')
        self.areaId = identifier
        self.area = area

    def getArea(self):
        return self.area

    def getAreaId(self):
        return self.areaId


class SelectNotify (Notify):
    """SelectNotify should be sent when something gets selected.  This
    can be for example an item in a list, or an icon."""
    
    def __init__(self, id, newSelection, doubleClicked=False):
        """Construct a new selection notification.

        @param id Identifies where the selection has occured.  The
        suffix '-selected' will be automatically appended.

        @param newSelection Identifier of the item that is now selected.

        @param doubleClicked True, if the selection was done using a
        double click.
        """
        Notify.__init__(self, id + "-selected")
        self.selection = newSelection
        self.doubleClick = doubleClicked

    def getSelection(self):
        """Returns the identifier of the item that is now selected."""
        return self.selection

    def isDoubleClick(self):
        return self.doubleClick


class DeselectNotify (Notify):
    """DeselectNotify is sent when something is deselected."""

    def __init__(self, id, deselectedItemIdentifier):
        """Construct a new deselection notification.

        @param id  Identifies where the deselection has occured.

        @param deselectedItemIdentifer  Deselected item.
        """
        Notify.__init__(self, id + '-deselected')
        self.deselection = deselectedItemIdentifier

    def getDeselection(self):
        return self.deselection


class FocusNotify (Notify):
    """FocusNotify should be sent when something gains the input
    focus.  Not all widgets send this, though."""

    def __init__(self, id):
        """Construct a new focus change event.
        @param id Identifier of the object that now has the focus.
        """
        Notify.__init__(self, "focus-changed")
        self.focus = id

    def getFocus(self):
        """Return the identifier of whatever now has the focus."""
        return self.focus


class FocusRequestNotify (FocusNotify):
    def __init__(self, id):
        FocusNotify.__init__(self, id)
        self.id = 'focus-request'


class ValueNotify (Notify):
    """ValueNotify is sent by a profile when a profiles.Value in it
    changes its value."""

    def __init__(self, settingId, newValue, profile):
        Notify.__init__(self, "value-changed")
        self.settingId = settingId
        self.newValue = newValue
        self.profile = profile

    def getSetting(self):
        return self.settingId

    def getValue(self):
        return self.newValue

    def getProfile(self):
        return self.profile
        
    def getTargetedEvent(self):
        """Make a new event that is targetable to a specific handler."""
        
        ev = ValueNotify(self.settingId, self.newValue, self.profile)
        ev.id = self.settingId + '-value-changed'
        return ev
    

class EditNotify (Notify):
    """EditNotify must be sent when an editable widget changes its
    value.  The owner of the widget can then react to the change."""

    def __init__(self, id, newValue):
        Notify.__init__(self, "widget-edited")
        self.widget = id
        self.newValue = newValue

    def getWidget(self):
        return self.widget

    def getValue(self):
        return self.newValue


class AttachNotify (Notify):
    """An AttachNotify is sent when an addon is attached or detached
    from a profile."""

    def __init__(self, addonIdentifier, profile, isAttached=True):
        if isAttached:
            eventId = 'addon-attached'
        else:
            eventId = 'addon-detached'
        Notify.__init__(self, eventId)
        self.addon = addonIdentifier
        self.profile = profile

    def getAddon(self):
        return self.addon

    def getProfile(self):
        return self.profile

    
class ProfileNotify (Notify):
    """ProfileNotify is sent when a profile's information has changed
    or a new profile has been loaded."""
    
    def __init__(self, profile):
        Notify.__init__(self, 'profile-updated')
        self.profile = profile
    
    def getProfile(self):
        return self.profile


class ActiveProfileNotify (Notify):
    """ActiveProfileNotify is sent when the active profile is changed,
    or when the active profile is refreshed."""
    
    def __init__(self, profile, wasChanged=True):
        Notify.__init__(self, 'active-profile-changed')
        self.profile = profile
        self.changed = wasChanged
        
    def getProfile(self):
        return self.profile
        
    def wasChanged(self):
        return self.changed


class LaunchNotify (Notify):
    """LaunchNotify is sent when the command line options for
    launching a game has been compiled."""

    def __init__(self, commandLineOptions):
        Notify.__init__(self, 'launching')
        self.options = commandLineOptions

    def getOptions(self):
        """This is the actual list of parameters that will be used for
        launching the game.  Notification listeners are permitted to
        edit the options, but it should be noted that there are no
        guarantees as to which listener gets to handle the options
        first.

        @return An array of command line parameter strings.
        """
        return self.options


class LanguageNotify (Notify):
    """A LanguageNotify is sent when the user interface language is
    changed."""

    def __init__(self, newLanguage):
        Notify.__init__(self, 'language-changed')
        # We'll assume the language identifier begins with 'language-'.
        self.lang = newLanguage[9:]

    def getLanguage(self):
        return self.lang


#class AddonNotify (Notify):
#    """AddonNotify is sent when an addon is installed or uninstalled.
#    Plugins should refresh their widgets to reflect the new
#    configuration."""##
#
#    def __init__(self, 


def addListener(listeners, eventIds, callback):
    if eventIds == None:
        # General purpose callback.
        listeners[None].append(callback)
        #print "General purpose callback " + str(callback)
        
    # No duplicates allowed.
    elif callback not in listeners[None]:
        for id in eventIds:
            if not listeners.has_key(id):
                listeners[id] = [callback]
            else:
                if callback not in listeners[id]:
                    listeners[id].append(callback)
                #else:
                #    print "Duplicate addition of listener " + str(callback) + " into " + id


def addCommandListener(callback, eventIds = None):
    """Register a new Command listener callback function.  When a
    Command event is sent, the callback will get called.

    @param callback Callback function for Commands.
    @param eventIds List of event identifiers.
    """
    addListener(commandListeners, eventIds, callback)


def addNotifyListener(callback, eventIds = None):
    """Register a new Notify listener callback function.  When a
    Notify event is sent, the callback will get called.

    @param callback Callback function for Notifys.
    @param eventIds List of event identifiers.
    """
    addListener(notifyListeners, eventIds, callback)


def removeCommandListener(callback):
    """Remove an existing Command listener callback function.

    @param callback Callback function for Commands.
    """
    for key in commandListeners.keys():
        try:
            commandListeners[key].remove(callback)
        except:
            # Not necessarily in this list.
            pass
        
        
def removeNotifyListener(callback):
    """Remove an existing Notify listener callback function.

    @param callback Callback function for Notifys.
    """
    for key in notifyListeners.keys():
        try:
            notifyListeners[key].remove(callback)
        except:
            # Not necessarily in this list.
            pass


def mute():
    """Mutes the sending of events. While muted, all events are discarded."""
    global areEventsMuted
    areEventsMuted = True


def unmute():
    """Unmutes the sending of events. While muted, all events are discarded."""
    global areEventsMuted
    areEventsMuted = False
    
    
def isMuted():
    return areEventsMuted

        
def send(event):
    """Broadcast an event to all listeners of the appropriate type.
    The event is processed synchronously: all listeners will be
    notified immediately during the execution of this function.

    @param event The event to send.  All listeners will get the same
    event object.
    """
    if areEventsMuted:
        return # Discard.

    global sendDepth, queuedEvents

    sendDepth += 1
    
    # Commands and Notifys go to a different set of listeners.
    if event.myClass == Command:
        listeners = commandListeners
    else:
        listeners = notifyListeners

    # The identifier of the event to be sent.
    sendId = event.getId()
    
    # Always include the unfiltered callbacks.
    callbacks = [c for c in listeners[None]]
    
    if listeners.has_key(sendId):
        # The specialized callbacks.
        callbacks += listeners[sendId]

    #print "Sending " + sendId + ":"
    #print callbacks

    # Send the event to all the appropriate listeners.
    for callback in callbacks:
        try:
            callback(event)
        except Exception, x:
			# Report errors.
            import logger
            logger.add(logger.HIGH, 'error-runtime-exception-during-event',
					   sendId, str(x), logger.formatTraceback())
            logger.show()

    # If event processing is complete but events are queued, send them
    # now.
    if sendDepth == 1:
        while len(queuedEvents) > 0:
            # Listeners may call sendAfter() during this while loop.
            roster = queuedEvents
            queuedEvents = []
            for ev in roster:
                send(ev)

    sendDepth -= 1
    
    # If a value-changed event is sent, also send the specially targeted
    # event meant to be handled by the widget who handles the value.
    if event.hasId('value-changed'):
        # Place it in the beginning of the queue.
        send(event.getTargetedEvent())


def sendAfter(event):
    """Broadcast an event to all listeners of the appropriate type.
    If this is called during the execution of send(), the event will
    be delayed until send() is finished.

    @param event The event to send.
    """
    if areEventsMuted:
        return # Discard.
    
    global sendDepth, queuedEvents

    if sendDepth > 0:
        # A send() is currently underway.  Queue the event.
        queuedEvents.append(event)
    else:
        # We can send immediately.
        send(event)
